import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

Boolean = bool
BoxedBoolean = bool
BoxedInteger = int
Capacity = int
ClusterName = str
DescribeAddonVersionsRequestMaxResults = int
FargateProfilesRequestMaxResults = int
ListAddonsRequestMaxResults = int
ListClustersRequestMaxResults = int
ListIdentityProviderConfigsRequestMaxResults = int
ListNodegroupsRequestMaxResults = int
ListUpdatesRequestMaxResults = int
NonZeroInteger = int
PercentCapacity = int
RoleArn = str
String = str
TagKey = str
TagValue = str
ZeroCapacity = int
labelKey = str
labelValue = str
requiredClaimsKey = str
requiredClaimsValue = str
taintKey = str
taintValue = str


class AMITypes(str):
    AL2_x86_64 = "AL2_x86_64"
    AL2_x86_64_GPU = "AL2_x86_64_GPU"
    AL2_ARM_64 = "AL2_ARM_64"
    CUSTOM = "CUSTOM"
    BOTTLEROCKET_ARM_64 = "BOTTLEROCKET_ARM_64"
    BOTTLEROCKET_x86_64 = "BOTTLEROCKET_x86_64"
    BOTTLEROCKET_ARM_64_NVIDIA = "BOTTLEROCKET_ARM_64_NVIDIA"
    BOTTLEROCKET_x86_64_NVIDIA = "BOTTLEROCKET_x86_64_NVIDIA"


class AddonIssueCode(str):
    AccessDenied = "AccessDenied"
    InternalFailure = "InternalFailure"
    ClusterUnreachable = "ClusterUnreachable"
    InsufficientNumberOfReplicas = "InsufficientNumberOfReplicas"
    ConfigurationConflict = "ConfigurationConflict"
    AdmissionRequestDenied = "AdmissionRequestDenied"
    UnsupportedAddonModification = "UnsupportedAddonModification"
    K8sResourceNotFound = "K8sResourceNotFound"


class AddonStatus(str):
    CREATING = "CREATING"
    ACTIVE = "ACTIVE"
    CREATE_FAILED = "CREATE_FAILED"
    UPDATING = "UPDATING"
    DELETING = "DELETING"
    DELETE_FAILED = "DELETE_FAILED"
    DEGRADED = "DEGRADED"


class CapacityTypes(str):
    ON_DEMAND = "ON_DEMAND"
    SPOT = "SPOT"


class ClusterStatus(str):
    CREATING = "CREATING"
    ACTIVE = "ACTIVE"
    DELETING = "DELETING"
    FAILED = "FAILED"
    UPDATING = "UPDATING"
    PENDING = "PENDING"


class ConnectorConfigProvider(str):
    EKS_ANYWHERE = "EKS_ANYWHERE"
    ANTHOS = "ANTHOS"
    GKE = "GKE"
    AKS = "AKS"
    OPENSHIFT = "OPENSHIFT"
    TANZU = "TANZU"
    RANCHER = "RANCHER"
    EC2 = "EC2"
    OTHER = "OTHER"


class ErrorCode(str):
    SubnetNotFound = "SubnetNotFound"
    SecurityGroupNotFound = "SecurityGroupNotFound"
    EniLimitReached = "EniLimitReached"
    IpNotAvailable = "IpNotAvailable"
    AccessDenied = "AccessDenied"
    OperationNotPermitted = "OperationNotPermitted"
    VpcIdNotFound = "VpcIdNotFound"
    Unknown = "Unknown"
    NodeCreationFailure = "NodeCreationFailure"
    PodEvictionFailure = "PodEvictionFailure"
    InsufficientFreeAddresses = "InsufficientFreeAddresses"
    ClusterUnreachable = "ClusterUnreachable"
    InsufficientNumberOfReplicas = "InsufficientNumberOfReplicas"
    ConfigurationConflict = "ConfigurationConflict"
    AdmissionRequestDenied = "AdmissionRequestDenied"
    UnsupportedAddonModification = "UnsupportedAddonModification"
    K8sResourceNotFound = "K8sResourceNotFound"


class FargateProfileStatus(str):
    CREATING = "CREATING"
    ACTIVE = "ACTIVE"
    DELETING = "DELETING"
    CREATE_FAILED = "CREATE_FAILED"
    DELETE_FAILED = "DELETE_FAILED"


class IpFamily(str):
    ipv4 = "ipv4"
    ipv6 = "ipv6"


class LogType(str):
    api = "api"
    audit = "audit"
    authenticator = "authenticator"
    controllerManager = "controllerManager"
    scheduler = "scheduler"


class NodegroupIssueCode(str):
    AutoScalingGroupNotFound = "AutoScalingGroupNotFound"
    AutoScalingGroupInvalidConfiguration = "AutoScalingGroupInvalidConfiguration"
    Ec2SecurityGroupNotFound = "Ec2SecurityGroupNotFound"
    Ec2SecurityGroupDeletionFailure = "Ec2SecurityGroupDeletionFailure"
    Ec2LaunchTemplateNotFound = "Ec2LaunchTemplateNotFound"
    Ec2LaunchTemplateVersionMismatch = "Ec2LaunchTemplateVersionMismatch"
    Ec2SubnetNotFound = "Ec2SubnetNotFound"
    Ec2SubnetInvalidConfiguration = "Ec2SubnetInvalidConfiguration"
    IamInstanceProfileNotFound = "IamInstanceProfileNotFound"
    IamLimitExceeded = "IamLimitExceeded"
    IamNodeRoleNotFound = "IamNodeRoleNotFound"
    NodeCreationFailure = "NodeCreationFailure"
    AsgInstanceLaunchFailures = "AsgInstanceLaunchFailures"
    InstanceLimitExceeded = "InstanceLimitExceeded"
    InsufficientFreeAddresses = "InsufficientFreeAddresses"
    AccessDenied = "AccessDenied"
    InternalFailure = "InternalFailure"
    ClusterUnreachable = "ClusterUnreachable"
    Ec2SubnetMissingIpv6Assignment = "Ec2SubnetMissingIpv6Assignment"


class NodegroupStatus(str):
    CREATING = "CREATING"
    ACTIVE = "ACTIVE"
    UPDATING = "UPDATING"
    DELETING = "DELETING"
    CREATE_FAILED = "CREATE_FAILED"
    DELETE_FAILED = "DELETE_FAILED"
    DEGRADED = "DEGRADED"


class ResolveConflicts(str):
    OVERWRITE = "OVERWRITE"
    NONE = "NONE"


class TaintEffect(str):
    NO_SCHEDULE = "NO_SCHEDULE"
    NO_EXECUTE = "NO_EXECUTE"
    PREFER_NO_SCHEDULE = "PREFER_NO_SCHEDULE"


class UpdateParamType(str):
    Version = "Version"
    PlatformVersion = "PlatformVersion"
    EndpointPrivateAccess = "EndpointPrivateAccess"
    EndpointPublicAccess = "EndpointPublicAccess"
    ClusterLogging = "ClusterLogging"
    DesiredSize = "DesiredSize"
    LabelsToAdd = "LabelsToAdd"
    LabelsToRemove = "LabelsToRemove"
    TaintsToAdd = "TaintsToAdd"
    TaintsToRemove = "TaintsToRemove"
    MaxSize = "MaxSize"
    MinSize = "MinSize"
    ReleaseVersion = "ReleaseVersion"
    PublicAccessCidrs = "PublicAccessCidrs"
    LaunchTemplateName = "LaunchTemplateName"
    LaunchTemplateVersion = "LaunchTemplateVersion"
    IdentityProviderConfig = "IdentityProviderConfig"
    EncryptionConfig = "EncryptionConfig"
    AddonVersion = "AddonVersion"
    ServiceAccountRoleArn = "ServiceAccountRoleArn"
    ResolveConflicts = "ResolveConflicts"
    MaxUnavailable = "MaxUnavailable"
    MaxUnavailablePercentage = "MaxUnavailablePercentage"


class UpdateStatus(str):
    InProgress = "InProgress"
    Failed = "Failed"
    Cancelled = "Cancelled"
    Successful = "Successful"


class UpdateType(str):
    VersionUpdate = "VersionUpdate"
    EndpointAccessUpdate = "EndpointAccessUpdate"
    LoggingUpdate = "LoggingUpdate"
    ConfigUpdate = "ConfigUpdate"
    AssociateIdentityProviderConfig = "AssociateIdentityProviderConfig"
    DisassociateIdentityProviderConfig = "DisassociateIdentityProviderConfig"
    AssociateEncryptionConfig = "AssociateEncryptionConfig"
    AddonUpdate = "AddonUpdate"


class configStatus(str):
    CREATING = "CREATING"
    DELETING = "DELETING"
    ACTIVE = "ACTIVE"


class AccessDeniedException(ServiceException):
    """You don't have permissions to perform the requested operation. The user
    or role that is making the request must have at least one IAM
    permissions policy attached that grants the required permissions. For
    more information, see `Access
    Management <https://docs.aws.amazon.com/IAM/latest/UserGuide/access.html>`__
    in the *IAM User Guide*.
    """

    message: Optional[String]


class BadRequestException(ServiceException):
    """This exception is thrown if the request contains a semantic error. The
    precise meaning will depend on the API, and will be documented in the
    error message.
    """

    message: Optional[String]


class ClientException(ServiceException):
    """These errors are usually caused by a client action. Actions can include
    using an action or resource on behalf of a user that doesn't have
    permissions to use the action or resource or specifying an identifier
    that is not valid.
    """

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class InvalidParameterException(ServiceException):
    """The specified parameter is invalid. Review the available parameters for
    the API request.
    """

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    fargateProfileName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class InvalidRequestException(ServiceException):
    """The request is invalid given the state of the cluster. Check the state
    of the cluster and the associated operations.
    """

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class NotFoundException(ServiceException):
    """A service resource associated with the request could not be found.
    Clients should not retry such requests.
    """

    message: Optional[String]


class ResourceInUseException(ServiceException):
    """The specified resource is in use."""

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class ResourceLimitExceededException(ServiceException):
    """You have encountered a service limit on the specified resource."""

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    message: Optional[String]


class ResourceNotFoundException(ServiceException):
    """The specified resource could not be found. You can view your available
    clusters with ListClusters. You can view your available managed node
    groups with ListNodegroups. Amazon EKS clusters and node groups are
    Region-specific.
    """

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    fargateProfileName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class ResourcePropagationDelayException(ServiceException):
    """Required resources (such as service-linked roles) were created and are
    still propagating. Retry later.
    """

    message: Optional[String]


class ServerException(ServiceException):
    """These errors are usually caused by a server-side issue."""

    clusterName: Optional[String]
    nodegroupName: Optional[String]
    addonName: Optional[String]
    message: Optional[String]


class ServiceUnavailableException(ServiceException):
    """The service is unavailable. Back off and retry the operation."""

    message: Optional[String]


StringList = List[String]


class UnsupportedAvailabilityZoneException(ServiceException):
    """At least one of your specified cluster subnets is in an Availability
    Zone that does not support Amazon EKS. The exception output specifies
    the supported Availability Zones for your account, from which you can
    choose subnets for your cluster.
    """

    message: Optional[String]
    clusterName: Optional[String]
    nodegroupName: Optional[String]
    validZones: Optional[StringList]


TagMap = Dict[TagKey, TagValue]
Timestamp = datetime


class AddonIssue(TypedDict, total=False):
    """An issue related to an add-on."""

    code: Optional[AddonIssueCode]
    message: Optional[String]
    resourceIds: Optional[StringList]


AddonIssueList = List[AddonIssue]


class AddonHealth(TypedDict, total=False):
    """The health of the add-on."""

    issues: Optional[AddonIssueList]


class Addon(TypedDict, total=False):
    """An Amazon EKS add-on. For more information, see `Amazon EKS
    add-ons <https://docs.aws.amazon.com/eks/latest/userguide/eks-add-ons.html>`__
    in the *Amazon EKS User Guide*.
    """

    addonName: Optional[String]
    clusterName: Optional[ClusterName]
    status: Optional[AddonStatus]
    addonVersion: Optional[String]
    health: Optional[AddonHealth]
    addonArn: Optional[String]
    createdAt: Optional[Timestamp]
    modifiedAt: Optional[Timestamp]
    serviceAccountRoleArn: Optional[String]
    tags: Optional[TagMap]


class Compatibility(TypedDict, total=False):
    """Compatibility information."""

    clusterVersion: Optional[String]
    platformVersions: Optional[StringList]
    defaultVersion: Optional[Boolean]


Compatibilities = List[Compatibility]


class AddonVersionInfo(TypedDict, total=False):
    """Information about an add-on version."""

    addonVersion: Optional[String]
    architecture: Optional[StringList]
    compatibilities: Optional[Compatibilities]


AddonVersionInfoList = List[AddonVersionInfo]
AddonInfo = TypedDict(
    "AddonInfo",
    {
        "addonName": Optional[String],
        "type": Optional[String],
        "addonVersions": Optional[AddonVersionInfoList],
    },
    total=False,
)
Addons = List[AddonInfo]


class Provider(TypedDict, total=False):
    """Identifies the Key Management Service (KMS) key used to encrypt the
    secrets.
    """

    keyArn: Optional[String]


class EncryptionConfig(TypedDict, total=False):
    """The encryption configuration for the cluster."""

    resources: Optional[StringList]
    provider: Optional[Provider]


EncryptionConfigList = List[EncryptionConfig]


class AssociateEncryptionConfigRequest(ServiceRequest):
    clusterName: String
    encryptionConfig: EncryptionConfigList
    clientRequestToken: Optional[String]


class ErrorDetail(TypedDict, total=False):
    """An object representing an error when an asynchronous operation fails."""

    errorCode: Optional[ErrorCode]
    errorMessage: Optional[String]
    resourceIds: Optional[StringList]


ErrorDetails = List[ErrorDetail]
UpdateParam = TypedDict(
    "UpdateParam",
    {
        "type": Optional[UpdateParamType],
        "value": Optional[String],
    },
    total=False,
)
UpdateParams = List[UpdateParam]
Update = TypedDict(
    "Update",
    {
        "id": Optional[String],
        "status": Optional[UpdateStatus],
        "type": Optional[UpdateType],
        "params": Optional[UpdateParams],
        "createdAt": Optional[Timestamp],
        "errors": Optional[ErrorDetails],
    },
    total=False,
)


class AssociateEncryptionConfigResponse(TypedDict, total=False):
    update: Optional[Update]


requiredClaimsMap = Dict[requiredClaimsKey, requiredClaimsValue]


class OidcIdentityProviderConfigRequest(TypedDict, total=False):
    """An object representing an OpenID Connect (OIDC) configuration. Before
    associating an OIDC identity provider to your cluster, review the
    considerations in `Authenticating users for your cluster from an OpenID
    Connect identity
    provider <https://docs.aws.amazon.com/eks/latest/userguide/authenticate-oidc-identity-provider.html>`__
    in the *Amazon EKS User Guide*.
    """

    identityProviderConfigName: String
    issuerUrl: String
    clientId: String
    usernameClaim: Optional[String]
    usernamePrefix: Optional[String]
    groupsClaim: Optional[String]
    groupsPrefix: Optional[String]
    requiredClaims: Optional[requiredClaimsMap]


class AssociateIdentityProviderConfigRequest(ServiceRequest):
    clusterName: String
    oidc: OidcIdentityProviderConfigRequest
    tags: Optional[TagMap]
    clientRequestToken: Optional[String]


class AssociateIdentityProviderConfigResponse(TypedDict, total=False):
    update: Optional[Update]
    tags: Optional[TagMap]


class AutoScalingGroup(TypedDict, total=False):
    """An Auto Scaling group that is associated with an Amazon EKS managed node
    group.
    """

    name: Optional[String]


AutoScalingGroupList = List[AutoScalingGroup]


class Certificate(TypedDict, total=False):
    """An object representing the ``certificate-authority-data`` for your
    cluster.
    """

    data: Optional[String]


class ConnectorConfigResponse(TypedDict, total=False):
    """The full description of your connected cluster."""

    activationId: Optional[String]
    activationCode: Optional[String]
    activationExpiry: Optional[Timestamp]
    provider: Optional[String]
    roleArn: Optional[String]


class OIDC(TypedDict, total=False):
    """An object representing the `OpenID
    Connect <https://openid.net/connect/>`__ (OIDC) identity provider
    information for the cluster.
    """

    issuer: Optional[String]


class Identity(TypedDict, total=False):
    """An object representing an identity provider."""

    oidc: Optional[OIDC]


LogTypes = List[LogType]


class LogSetup(TypedDict, total=False):
    """An object representing the enabled or disabled Kubernetes control plane
    logs for your cluster.
    """

    types: Optional[LogTypes]
    enabled: Optional[BoxedBoolean]


LogSetups = List[LogSetup]


class Logging(TypedDict, total=False):
    """An object representing the logging configuration for resources in your
    cluster.
    """

    clusterLogging: Optional[LogSetups]


class KubernetesNetworkConfigResponse(TypedDict, total=False):
    """The Kubernetes network configuration for the cluster. The response
    contains a value for **serviceIpv6Cidr** or **serviceIpv4Cidr**, but not
    both.
    """

    serviceIpv4Cidr: Optional[String]
    serviceIpv6Cidr: Optional[String]
    ipFamily: Optional[IpFamily]


class VpcConfigResponse(TypedDict, total=False):
    """An object representing an Amazon EKS cluster VPC configuration response."""

    subnetIds: Optional[StringList]
    securityGroupIds: Optional[StringList]
    clusterSecurityGroupId: Optional[String]
    vpcId: Optional[String]
    endpointPublicAccess: Optional[Boolean]
    endpointPrivateAccess: Optional[Boolean]
    publicAccessCidrs: Optional[StringList]


class Cluster(TypedDict, total=False):
    """An object representing an Amazon EKS cluster."""

    name: Optional[String]
    arn: Optional[String]
    createdAt: Optional[Timestamp]
    version: Optional[String]
    endpoint: Optional[String]
    roleArn: Optional[String]
    resourcesVpcConfig: Optional[VpcConfigResponse]
    kubernetesNetworkConfig: Optional[KubernetesNetworkConfigResponse]
    logging: Optional[Logging]
    identity: Optional[Identity]
    status: Optional[ClusterStatus]
    certificateAuthority: Optional[Certificate]
    clientRequestToken: Optional[String]
    platformVersion: Optional[String]
    tags: Optional[TagMap]
    encryptionConfig: Optional[EncryptionConfigList]
    connectorConfig: Optional[ConnectorConfigResponse]


class ConnectorConfigRequest(TypedDict, total=False):
    """The configuration sent to a cluster for configuration."""

    roleArn: String
    provider: ConnectorConfigProvider


class CreateAddonRequest(ServiceRequest):
    clusterName: ClusterName
    addonName: String
    addonVersion: Optional[String]
    serviceAccountRoleArn: Optional[RoleArn]
    resolveConflicts: Optional[ResolveConflicts]
    clientRequestToken: Optional[String]
    tags: Optional[TagMap]


class CreateAddonResponse(TypedDict, total=False):
    addon: Optional[Addon]


class KubernetesNetworkConfigRequest(TypedDict, total=False):
    """The Kubernetes network configuration for the cluster."""

    serviceIpv4Cidr: Optional[String]
    ipFamily: Optional[IpFamily]


class VpcConfigRequest(TypedDict, total=False):
    """An object representing the VPC configuration to use for an Amazon EKS
    cluster.
    """

    subnetIds: Optional[StringList]
    securityGroupIds: Optional[StringList]
    endpointPublicAccess: Optional[BoxedBoolean]
    endpointPrivateAccess: Optional[BoxedBoolean]
    publicAccessCidrs: Optional[StringList]


class CreateClusterRequest(ServiceRequest):
    name: ClusterName
    version: Optional[String]
    roleArn: String
    resourcesVpcConfig: VpcConfigRequest
    kubernetesNetworkConfig: Optional[KubernetesNetworkConfigRequest]
    logging: Optional[Logging]
    clientRequestToken: Optional[String]
    tags: Optional[TagMap]
    encryptionConfig: Optional[EncryptionConfigList]


class CreateClusterResponse(TypedDict, total=False):
    cluster: Optional[Cluster]


FargateProfileLabel = Dict[String, String]


class FargateProfileSelector(TypedDict, total=False):
    """An object representing an Fargate profile selector."""

    namespace: Optional[String]
    labels: Optional[FargateProfileLabel]


FargateProfileSelectors = List[FargateProfileSelector]


class CreateFargateProfileRequest(ServiceRequest):
    fargateProfileName: String
    clusterName: String
    podExecutionRoleArn: String
    subnets: Optional[StringList]
    selectors: Optional[FargateProfileSelectors]
    clientRequestToken: Optional[String]
    tags: Optional[TagMap]


class FargateProfile(TypedDict, total=False):
    """An object representing an Fargate profile."""

    fargateProfileName: Optional[String]
    fargateProfileArn: Optional[String]
    clusterName: Optional[String]
    createdAt: Optional[Timestamp]
    podExecutionRoleArn: Optional[String]
    subnets: Optional[StringList]
    selectors: Optional[FargateProfileSelectors]
    status: Optional[FargateProfileStatus]
    tags: Optional[TagMap]


class CreateFargateProfileResponse(TypedDict, total=False):
    fargateProfile: Optional[FargateProfile]


class NodegroupUpdateConfig(TypedDict, total=False):
    """The node group update configuration."""

    maxUnavailable: Optional[NonZeroInteger]
    maxUnavailablePercentage: Optional[PercentCapacity]


class LaunchTemplateSpecification(TypedDict, total=False):
    """An object representing a node group launch template specification. The
    launch template cannot include
    ```SubnetId`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateNetworkInterface.html>`__
    ,
    ```IamInstanceProfile`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_IamInstanceProfile.html>`__
    ,
    ```RequestSpotInstances`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RequestSpotInstances.html>`__
    ,
    ```HibernationOptions`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_HibernationOptionsRequest.html>`__
    , or
    ```TerminateInstances`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_TerminateInstances.html>`__
    , or the node group deployment or update will fail. For more information
    about launch templates, see
    ```CreateLaunchTemplate`` <https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateLaunchTemplate.html>`__
    in the Amazon EC2 API Reference. For more information about using launch
    templates with Amazon EKS, see `Launch template
    support <https://docs.aws.amazon.com/eks/latest/userguide/launch-templates.html>`__
    in the *Amazon EKS User Guide*.

    Specify either ``name`` or ``id``, but not both.
    """

    name: Optional[String]
    version: Optional[String]
    id: Optional[String]


class Taint(TypedDict, total=False):
    """A property that allows a node to repel a set of pods. For more
    information, see `Node taints on managed node
    groups <https://docs.aws.amazon.com/eks/latest/userguide/node-taints-managed-node-groups.html>`__.
    """

    key: Optional[taintKey]
    value: Optional[taintValue]
    effect: Optional[TaintEffect]


taintsList = List[Taint]
labelsMap = Dict[labelKey, labelValue]


class RemoteAccessConfig(TypedDict, total=False):
    """An object representing the remote access configuration for the managed
    node group.
    """

    ec2SshKey: Optional[String]
    sourceSecurityGroups: Optional[StringList]


class NodegroupScalingConfig(TypedDict, total=False):
    """An object representing the scaling configuration details for the Auto
    Scaling group that is associated with your node group. When creating a
    node group, you must specify all or none of the properties. When
    updating a node group, you can specify any or none of the properties.
    """

    minSize: Optional[ZeroCapacity]
    maxSize: Optional[Capacity]
    desiredSize: Optional[ZeroCapacity]


class CreateNodegroupRequest(ServiceRequest):
    clusterName: String
    nodegroupName: String
    scalingConfig: Optional[NodegroupScalingConfig]
    diskSize: Optional[BoxedInteger]
    subnets: StringList
    instanceTypes: Optional[StringList]
    amiType: Optional[AMITypes]
    remoteAccess: Optional[RemoteAccessConfig]
    nodeRole: String
    labels: Optional[labelsMap]
    taints: Optional[taintsList]
    tags: Optional[TagMap]
    clientRequestToken: Optional[String]
    launchTemplate: Optional[LaunchTemplateSpecification]
    updateConfig: Optional[NodegroupUpdateConfig]
    capacityType: Optional[CapacityTypes]
    version: Optional[String]
    releaseVersion: Optional[String]


class Issue(TypedDict, total=False):
    """An object representing an issue with an Amazon EKS resource."""

    code: Optional[NodegroupIssueCode]
    message: Optional[String]
    resourceIds: Optional[StringList]


IssueList = List[Issue]


class NodegroupHealth(TypedDict, total=False):
    """An object representing the health status of the node group."""

    issues: Optional[IssueList]


class NodegroupResources(TypedDict, total=False):
    """An object representing the resources associated with the node group,
    such as Auto Scaling groups and security groups for remote access.
    """

    autoScalingGroups: Optional[AutoScalingGroupList]
    remoteAccessSecurityGroup: Optional[String]


class Nodegroup(TypedDict, total=False):
    """An object representing an Amazon EKS managed node group."""

    nodegroupName: Optional[String]
    nodegroupArn: Optional[String]
    clusterName: Optional[String]
    version: Optional[String]
    releaseVersion: Optional[String]
    createdAt: Optional[Timestamp]
    modifiedAt: Optional[Timestamp]
    status: Optional[NodegroupStatus]
    capacityType: Optional[CapacityTypes]
    scalingConfig: Optional[NodegroupScalingConfig]
    instanceTypes: Optional[StringList]
    subnets: Optional[StringList]
    remoteAccess: Optional[RemoteAccessConfig]
    amiType: Optional[AMITypes]
    nodeRole: Optional[String]
    labels: Optional[labelsMap]
    taints: Optional[taintsList]
    resources: Optional[NodegroupResources]
    diskSize: Optional[BoxedInteger]
    health: Optional[NodegroupHealth]
    updateConfig: Optional[NodegroupUpdateConfig]
    launchTemplate: Optional[LaunchTemplateSpecification]
    tags: Optional[TagMap]


class CreateNodegroupResponse(TypedDict, total=False):
    nodegroup: Optional[Nodegroup]


class DeleteAddonRequest(ServiceRequest):
    clusterName: ClusterName
    addonName: String
    preserve: Optional[Boolean]


class DeleteAddonResponse(TypedDict, total=False):
    addon: Optional[Addon]


class DeleteClusterRequest(ServiceRequest):
    name: String


class DeleteClusterResponse(TypedDict, total=False):
    cluster: Optional[Cluster]


class DeleteFargateProfileRequest(ServiceRequest):
    clusterName: String
    fargateProfileName: String


class DeleteFargateProfileResponse(TypedDict, total=False):
    fargateProfile: Optional[FargateProfile]


class DeleteNodegroupRequest(ServiceRequest):
    clusterName: String
    nodegroupName: String


class DeleteNodegroupResponse(TypedDict, total=False):
    nodegroup: Optional[Nodegroup]


class DeregisterClusterRequest(ServiceRequest):
    name: String


class DeregisterClusterResponse(TypedDict, total=False):
    cluster: Optional[Cluster]


class DescribeAddonRequest(ServiceRequest):
    clusterName: ClusterName
    addonName: String


class DescribeAddonResponse(TypedDict, total=False):
    addon: Optional[Addon]


class DescribeAddonVersionsRequest(ServiceRequest):
    kubernetesVersion: Optional[String]
    maxResults: Optional[DescribeAddonVersionsRequestMaxResults]
    nextToken: Optional[String]
    addonName: Optional[String]


class DescribeAddonVersionsResponse(TypedDict, total=False):
    addons: Optional[Addons]
    nextToken: Optional[String]


class DescribeClusterRequest(ServiceRequest):
    name: String


class DescribeClusterResponse(TypedDict, total=False):
    cluster: Optional[Cluster]


class DescribeFargateProfileRequest(ServiceRequest):
    clusterName: String
    fargateProfileName: String


class DescribeFargateProfileResponse(TypedDict, total=False):
    fargateProfile: Optional[FargateProfile]


IdentityProviderConfig = TypedDict(
    "IdentityProviderConfig",
    {
        "type": String,
        "name": String,
    },
    total=False,
)


class DescribeIdentityProviderConfigRequest(ServiceRequest):
    clusterName: String
    identityProviderConfig: IdentityProviderConfig


class OidcIdentityProviderConfig(TypedDict, total=False):
    """An object that represents the configuration for an OpenID Connect (OIDC)
    identity provider.
    """

    identityProviderConfigName: Optional[String]
    identityProviderConfigArn: Optional[String]
    clusterName: Optional[String]
    issuerUrl: Optional[String]
    clientId: Optional[String]
    usernameClaim: Optional[String]
    usernamePrefix: Optional[String]
    groupsClaim: Optional[String]
    groupsPrefix: Optional[String]
    requiredClaims: Optional[requiredClaimsMap]
    tags: Optional[TagMap]
    status: Optional[configStatus]


class IdentityProviderConfigResponse(TypedDict, total=False):
    """The full description of your identity configuration."""

    oidc: Optional[OidcIdentityProviderConfig]


class DescribeIdentityProviderConfigResponse(TypedDict, total=False):
    identityProviderConfig: Optional[IdentityProviderConfigResponse]


class DescribeNodegroupRequest(ServiceRequest):
    clusterName: String
    nodegroupName: String


class DescribeNodegroupResponse(TypedDict, total=False):
    nodegroup: Optional[Nodegroup]


class DescribeUpdateRequest(ServiceRequest):
    name: String
    updateId: String
    nodegroupName: Optional[String]
    addonName: Optional[String]


class DescribeUpdateResponse(TypedDict, total=False):
    update: Optional[Update]


class DisassociateIdentityProviderConfigRequest(ServiceRequest):
    clusterName: String
    identityProviderConfig: IdentityProviderConfig
    clientRequestToken: Optional[String]


class DisassociateIdentityProviderConfigResponse(TypedDict, total=False):
    update: Optional[Update]


IdentityProviderConfigs = List[IdentityProviderConfig]
IncludeClustersList = List[String]


class ListAddonsRequest(ServiceRequest):
    clusterName: ClusterName
    maxResults: Optional[ListAddonsRequestMaxResults]
    nextToken: Optional[String]


class ListAddonsResponse(TypedDict, total=False):
    addons: Optional[StringList]
    nextToken: Optional[String]


class ListClustersRequest(ServiceRequest):
    maxResults: Optional[ListClustersRequestMaxResults]
    nextToken: Optional[String]
    include: Optional[IncludeClustersList]


class ListClustersResponse(TypedDict, total=False):
    clusters: Optional[StringList]
    nextToken: Optional[String]


class ListFargateProfilesRequest(ServiceRequest):
    clusterName: String
    maxResults: Optional[FargateProfilesRequestMaxResults]
    nextToken: Optional[String]


class ListFargateProfilesResponse(TypedDict, total=False):
    fargateProfileNames: Optional[StringList]
    nextToken: Optional[String]


class ListIdentityProviderConfigsRequest(ServiceRequest):
    clusterName: String
    maxResults: Optional[ListIdentityProviderConfigsRequestMaxResults]
    nextToken: Optional[String]


class ListIdentityProviderConfigsResponse(TypedDict, total=False):
    identityProviderConfigs: Optional[IdentityProviderConfigs]
    nextToken: Optional[String]


class ListNodegroupsRequest(ServiceRequest):
    clusterName: String
    maxResults: Optional[ListNodegroupsRequestMaxResults]
    nextToken: Optional[String]


class ListNodegroupsResponse(TypedDict, total=False):
    nodegroups: Optional[StringList]
    nextToken: Optional[String]


class ListTagsForResourceRequest(ServiceRequest):
    resourceArn: String


class ListTagsForResourceResponse(TypedDict, total=False):
    tags: Optional[TagMap]


class ListUpdatesRequest(ServiceRequest):
    name: String
    nodegroupName: Optional[String]
    addonName: Optional[String]
    nextToken: Optional[String]
    maxResults: Optional[ListUpdatesRequestMaxResults]


class ListUpdatesResponse(TypedDict, total=False):
    updateIds: Optional[StringList]
    nextToken: Optional[String]


class RegisterClusterRequest(ServiceRequest):
    name: ClusterName
    connectorConfig: ConnectorConfigRequest
    clientRequestToken: Optional[String]
    tags: Optional[TagMap]


class RegisterClusterResponse(TypedDict, total=False):
    cluster: Optional[Cluster]


TagKeyList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    resourceArn: String
    tags: TagMap


class TagResourceResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    resourceArn: String
    tagKeys: TagKeyList


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateAddonRequest(ServiceRequest):
    clusterName: ClusterName
    addonName: String
    addonVersion: Optional[String]
    serviceAccountRoleArn: Optional[RoleArn]
    resolveConflicts: Optional[ResolveConflicts]
    clientRequestToken: Optional[String]


class UpdateAddonResponse(TypedDict, total=False):
    update: Optional[Update]


class UpdateClusterConfigRequest(ServiceRequest):
    name: String
    resourcesVpcConfig: Optional[VpcConfigRequest]
    logging: Optional[Logging]
    clientRequestToken: Optional[String]


class UpdateClusterConfigResponse(TypedDict, total=False):
    update: Optional[Update]


class UpdateClusterVersionRequest(ServiceRequest):
    name: String
    version: String
    clientRequestToken: Optional[String]


class UpdateClusterVersionResponse(TypedDict, total=False):
    update: Optional[Update]


labelsKeyList = List[String]


class UpdateLabelsPayload(TypedDict, total=False):
    """An object representing a Kubernetes label change for a managed node
    group.
    """

    addOrUpdateLabels: Optional[labelsMap]
    removeLabels: Optional[labelsKeyList]


class UpdateTaintsPayload(TypedDict, total=False):
    """An object representing the details of an update to a taints payload. For
    more information, see `Node taints on managed node
    groups <https://docs.aws.amazon.com/eks/latest/userguide/node-taints-managed-node-groups.html>`__.
    """

    addOrUpdateTaints: Optional[taintsList]
    removeTaints: Optional[taintsList]


class UpdateNodegroupConfigRequest(ServiceRequest):
    clusterName: String
    nodegroupName: String
    labels: Optional[UpdateLabelsPayload]
    taints: Optional[UpdateTaintsPayload]
    scalingConfig: Optional[NodegroupScalingConfig]
    updateConfig: Optional[NodegroupUpdateConfig]
    clientRequestToken: Optional[String]


class UpdateNodegroupConfigResponse(TypedDict, total=False):
    update: Optional[Update]


class UpdateNodegroupVersionRequest(ServiceRequest):
    clusterName: String
    nodegroupName: String
    version: Optional[String]
    releaseVersion: Optional[String]
    launchTemplate: Optional[LaunchTemplateSpecification]
    force: Optional[Boolean]
    clientRequestToken: Optional[String]


class UpdateNodegroupVersionResponse(TypedDict, total=False):
    update: Optional[Update]


class EksApi:

    service = "eks"
    version = "2017-11-01"

    @handler("AssociateEncryptionConfig")
    def associate_encryption_config(
        self,
        context: RequestContext,
        cluster_name: String,
        encryption_config: EncryptionConfigList,
        client_request_token: String = None,
    ) -> AssociateEncryptionConfigResponse:
        """Associate encryption configuration to an existing cluster.

        You can use this API to enable encryption on existing clusters which do
        not have encryption already enabled. This allows you to implement a
        defense-in-depth security strategy without migrating applications to new
        Amazon EKS clusters.

        :param cluster_name: The name of the cluster that you are associating with encryption
        configuration.
        :param encryption_config: The configuration you are using for encryption.
        :param client_request_token: The client request token you are using with the encryption
        configuration.
        :returns: AssociateEncryptionConfigResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("AssociateIdentityProviderConfig")
    def associate_identity_provider_config(
        self,
        context: RequestContext,
        cluster_name: String,
        oidc: OidcIdentityProviderConfigRequest,
        tags: TagMap = None,
        client_request_token: String = None,
    ) -> AssociateIdentityProviderConfigResponse:
        """Associate an identity provider configuration to a cluster.

        If you want to authenticate identities using an identity provider, you
        can create an identity provider configuration and associate it to your
        cluster. After configuring authentication to your cluster you can create
        Kubernetes ``roles`` and ``clusterroles`` to assign permissions to the
        roles, and then bind the roles to the identities using Kubernetes
        ``rolebindings`` and ``clusterrolebindings``. For more information see
        `Using RBAC
        Authorization <https://kubernetes.io/docs/reference/access-authn-authz/rbac/>`__
        in the Kubernetes documentation.

        :param cluster_name: The name of the cluster to associate the configuration to.
        :param oidc: An object that represents an OpenID Connect (OIDC) identity provider
        configuration.
        :param tags: The metadata to apply to the configuration to assist with categorization
        and organization.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: AssociateIdentityProviderConfigResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("CreateAddon")
    def create_addon(
        self,
        context: RequestContext,
        cluster_name: ClusterName,
        addon_name: String,
        addon_version: String = None,
        service_account_role_arn: RoleArn = None,
        resolve_conflicts: ResolveConflicts = None,
        client_request_token: String = None,
        tags: TagMap = None,
    ) -> CreateAddonResponse:
        """Creates an Amazon EKS add-on.

        Amazon EKS add-ons help to automate the provisioning and lifecycle
        management of common operational software for Amazon EKS clusters.
        Amazon EKS add-ons require clusters running version 1.18 or later
        because Amazon EKS add-ons rely on the Server-side Apply Kubernetes
        feature, which is only available in Kubernetes 1.18 and later. For more
        information, see `Amazon EKS
        add-ons <https://docs.aws.amazon.com/eks/latest/userguide/eks-add-ons.html>`__
        in the *Amazon EKS User Guide*.

        :param cluster_name: The name of the cluster to create the add-on for.
        :param addon_name: The name of the add-on.
        :param addon_version: The version of the add-on.
        :param service_account_role_arn: The Amazon Resource Name (ARN) of an existing IAM role to bind to the
        add-on's service account.
        :param resolve_conflicts: How to resolve parameter value conflicts when migrating an existing
        add-on to an Amazon EKS add-on.
        :param client_request_token: A unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param tags: The metadata to apply to the cluster to assist with categorization and
        organization.
        :returns: CreateAddonResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("CreateCluster")
    def create_cluster(
        self,
        context: RequestContext,
        name: ClusterName,
        role_arn: String,
        resources_vpc_config: VpcConfigRequest,
        version: String = None,
        kubernetes_network_config: KubernetesNetworkConfigRequest = None,
        logging: Logging = None,
        client_request_token: String = None,
        tags: TagMap = None,
        encryption_config: EncryptionConfigList = None,
    ) -> CreateClusterResponse:
        """Creates an Amazon EKS control plane.

        The Amazon EKS control plane consists of control plane instances that
        run the Kubernetes software, such as ``etcd`` and the API server. The
        control plane runs in an account managed by Amazon Web Services, and the
        Kubernetes API is exposed by the Amazon EKS API server endpoint. Each
        Amazon EKS cluster control plane is single tenant and unique. It runs on
        its own set of Amazon EC2 instances.

        The cluster control plane is provisioned across multiple Availability
        Zones and fronted by an Elastic Load Balancing Network Load Balancer.
        Amazon EKS also provisions elastic network interfaces in your VPC
        subnets to provide connectivity from the control plane instances to the
        nodes (for example, to support ``kubectl exec``, ``logs``, and ``proxy``
        data flows).

        Amazon EKS nodes run in your Amazon Web Services account and connect to
        your cluster's control plane over the Kubernetes API server endpoint and
        a certificate file that is created for your cluster.

        In most cases, it takes several minutes to create a cluster. After you
        create an Amazon EKS cluster, you must configure your Kubernetes tooling
        to communicate with the API server and launch nodes into your cluster.
        For more information, see `Managing Cluster
        Authentication <https://docs.aws.amazon.com/eks/latest/userguide/managing-auth.html>`__
        and `Launching Amazon EKS
        nodes <https://docs.aws.amazon.com/eks/latest/userguide/launch-workers.html>`__
        in the *Amazon EKS User Guide*.

        :param name: The unique name to give to your cluster.
        :param role_arn: The Amazon Resource Name (ARN) of the IAM role that provides permissions
        for the Kubernetes control plane to make calls to Amazon Web Services
        API operations on your behalf.
        :param resources_vpc_config: The VPC configuration that's used by the cluster control plane.
        :param version: The desired Kubernetes version for your cluster.
        :param kubernetes_network_config: The Kubernetes network configuration for the cluster.
        :param logging: Enable or disable exporting the Kubernetes control plane logs for your
        cluster to CloudWatch Logs.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param tags: The metadata to apply to the cluster to assist with categorization and
        organization.
        :param encryption_config: The encryption configuration for the cluster.
        :returns: CreateClusterResponse
        :raises ResourceInUseException:
        :raises ResourceLimitExceededException:
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        :raises UnsupportedAvailabilityZoneException:
        """
        raise NotImplementedError

    @handler("CreateFargateProfile")
    def create_fargate_profile(
        self,
        context: RequestContext,
        fargate_profile_name: String,
        cluster_name: String,
        pod_execution_role_arn: String,
        subnets: StringList = None,
        selectors: FargateProfileSelectors = None,
        client_request_token: String = None,
        tags: TagMap = None,
    ) -> CreateFargateProfileResponse:
        """Creates an Fargate profile for your Amazon EKS cluster. You must have at
        least one Fargate profile in a cluster to be able to run pods on
        Fargate.

        The Fargate profile allows an administrator to declare which pods run on
        Fargate and specify which pods run on which Fargate profile. This
        declaration is done through the profile’s selectors. Each profile can
        have up to five selectors that contain a namespace and labels. A
        namespace is required for every selector. The label field consists of
        multiple optional key-value pairs. Pods that match the selectors are
        scheduled on Fargate. If a to-be-scheduled pod matches any of the
        selectors in the Fargate profile, then that pod is run on Fargate.

        When you create a Fargate profile, you must specify a pod execution role
        to use with the pods that are scheduled with the profile. This role is
        added to the cluster's Kubernetes `Role Based Access
        Control <https://kubernetes.io/docs/admin/authorization/rbac/>`__ (RBAC)
        for authorization so that the ``kubelet`` that is running on the Fargate
        infrastructure can register with your Amazon EKS cluster so that it can
        appear in your cluster as a node. The pod execution role also provides
        IAM permissions to the Fargate infrastructure to allow read access to
        Amazon ECR image repositories. For more information, see `Pod Execution
        Role <https://docs.aws.amazon.com/eks/latest/userguide/pod-execution-role.html>`__
        in the *Amazon EKS User Guide*.

        Fargate profiles are immutable. However, you can create a new updated
        profile to replace an existing profile and then delete the original
        after the updated profile has finished creating.

        If any Fargate profiles in a cluster are in the ``DELETING`` status, you
        must wait for that Fargate profile to finish deleting before you can
        create any other profiles in that cluster.

        For more information, see `Fargate
        Profile <https://docs.aws.amazon.com/eks/latest/userguide/fargate-profile.html>`__
        in the *Amazon EKS User Guide*.

        :param fargate_profile_name: The name of the Fargate profile.
        :param cluster_name: The name of the Amazon EKS cluster to apply the Fargate profile to.
        :param pod_execution_role_arn: The Amazon Resource Name (ARN) of the pod execution role to use for pods
        that match the selectors in the Fargate profile.
        :param subnets: The IDs of subnets to launch your pods into.
        :param selectors: The selectors to match for pods to use this Fargate profile.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param tags: The metadata to apply to the Fargate profile to assist with
        categorization and organization.
        :returns: CreateFargateProfileResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceLimitExceededException:
        :raises UnsupportedAvailabilityZoneException:
        """
        raise NotImplementedError

    @handler("CreateNodegroup")
    def create_nodegroup(
        self,
        context: RequestContext,
        cluster_name: String,
        nodegroup_name: String,
        subnets: StringList,
        node_role: String,
        scaling_config: NodegroupScalingConfig = None,
        disk_size: BoxedInteger = None,
        instance_types: StringList = None,
        ami_type: AMITypes = None,
        remote_access: RemoteAccessConfig = None,
        labels: labelsMap = None,
        taints: taintsList = None,
        tags: TagMap = None,
        client_request_token: String = None,
        launch_template: LaunchTemplateSpecification = None,
        update_config: NodegroupUpdateConfig = None,
        capacity_type: CapacityTypes = None,
        version: String = None,
        release_version: String = None,
    ) -> CreateNodegroupResponse:
        """Creates a managed node group for an Amazon EKS cluster. You can only
        create a node group for your cluster that is equal to the current
        Kubernetes version for the cluster. All node groups are created with the
        latest AMI release version for the respective minor Kubernetes version
        of the cluster, unless you deploy a custom AMI using a launch template.
        For more information about using launch templates, see `Launch template
        support <https://docs.aws.amazon.com/eks/latest/userguide/launch-templates.html>`__.

        An Amazon EKS managed node group is an Amazon EC2 Auto Scaling group and
        associated Amazon EC2 instances that are managed by Amazon Web Services
        for an Amazon EKS cluster. Each node group uses a version of the Amazon
        EKS optimized Amazon Linux 2 AMI. For more information, see `Managed
        Node
        Groups <https://docs.aws.amazon.com/eks/latest/userguide/managed-node-groups.html>`__
        in the *Amazon EKS User Guide*.

        :param cluster_name: The name of the cluster to create the node group in.
        :param nodegroup_name: The unique name to give your node group.
        :param subnets: The subnets to use for the Auto Scaling group that is created for your
        node group.
        :param node_role: The Amazon Resource Name (ARN) of the IAM role to associate with your
        node group.
        :param scaling_config: The scaling configuration details for the Auto Scaling group that is
        created for your node group.
        :param disk_size: The root device disk size (in GiB) for your node group instances.
        :param instance_types: Specify the instance types for a node group.
        :param ami_type: The AMI type for your node group.
        :param remote_access: The remote access (SSH) configuration to use with your node group.
        :param labels: The Kubernetes labels to be applied to the nodes in the node group when
        they are created.
        :param taints: The Kubernetes taints to be applied to the nodes in the node group.
        :param tags: The metadata to apply to the node group to assist with categorization
        and organization.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param launch_template: An object representing a node group's launch template specification.
        :param update_config: The node group update configuration.
        :param capacity_type: The capacity type for your node group.
        :param version: The Kubernetes version to use for your managed nodes.
        :param release_version: The AMI version of the Amazon EKS optimized AMI to use with your node
        group.
        :returns: CreateNodegroupResponse
        :raises ResourceInUseException:
        :raises ResourceLimitExceededException:
        :raises InvalidRequestException:
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DeleteAddon")
    def delete_addon(
        self,
        context: RequestContext,
        cluster_name: ClusterName,
        addon_name: String,
        preserve: Boolean = None,
    ) -> DeleteAddonResponse:
        """Delete an Amazon EKS add-on.

        When you remove the add-on, it will also be deleted from the cluster.
        You can always manually start an add-on on the cluster using the
        Kubernetes API.

        :param cluster_name: The name of the cluster to delete the add-on from.
        :param addon_name: The name of the add-on.
        :param preserve: Specifying this option preserves the add-on software on your cluster but
        Amazon EKS stops managing any settings for the add-on.
        :returns: DeleteAddonResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DeleteCluster")
    def delete_cluster(self, context: RequestContext, name: String) -> DeleteClusterResponse:
        """Deletes the Amazon EKS cluster control plane.

        If you have active services in your cluster that are associated with a
        load balancer, you must delete those services before deleting the
        cluster so that the load balancers are deleted properly. Otherwise, you
        can have orphaned resources in your VPC that prevent you from being able
        to delete the VPC. For more information, see `Deleting a
        Cluster <https://docs.aws.amazon.com/eks/latest/userguide/delete-cluster.html>`__
        in the *Amazon EKS User Guide*.

        If you have managed node groups or Fargate profiles attached to the
        cluster, you must delete them first. For more information, see
        DeleteNodegroup and DeleteFargateProfile.

        :param name: The name of the cluster to delete.
        :returns: DeleteClusterResponse
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DeleteFargateProfile")
    def delete_fargate_profile(
        self, context: RequestContext, cluster_name: String, fargate_profile_name: String
    ) -> DeleteFargateProfileResponse:
        """Deletes an Fargate profile.

        When you delete a Fargate profile, any pods running on Fargate that were
        created with the profile are deleted. If those pods match another
        Fargate profile, then they are scheduled on Fargate with that profile.
        If they no longer match any Fargate profiles, then they are not
        scheduled on Fargate and they may remain in a pending state.

        Only one Fargate profile in a cluster can be in the ``DELETING`` status
        at a time. You must wait for a Fargate profile to finish deleting before
        you can delete any other profiles in that cluster.

        :param cluster_name: The name of the Amazon EKS cluster associated with the Fargate profile
        to delete.
        :param fargate_profile_name: The name of the Fargate profile to delete.
        :returns: DeleteFargateProfileResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("DeleteNodegroup")
    def delete_nodegroup(
        self, context: RequestContext, cluster_name: String, nodegroup_name: String
    ) -> DeleteNodegroupResponse:
        """Deletes an Amazon EKS node group for a cluster.

        :param cluster_name: The name of the Amazon EKS cluster that is associated with your node
        group.
        :param nodegroup_name: The name of the node group to delete.
        :returns: DeleteNodegroupResponse
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DeregisterCluster")
    def deregister_cluster(
        self, context: RequestContext, name: String
    ) -> DeregisterClusterResponse:
        """Deregisters a connected cluster to remove it from the Amazon EKS control
        plane.

        :param name: The name of the connected cluster to deregister.
        :returns: DeregisterClusterResponse
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        :raises AccessDeniedException:
        """
        raise NotImplementedError

    @handler("DescribeAddon")
    def describe_addon(
        self, context: RequestContext, cluster_name: ClusterName, addon_name: String
    ) -> DescribeAddonResponse:
        """Describes an Amazon EKS add-on.

        :param cluster_name: The name of the cluster.
        :param addon_name: The name of the add-on.
        :returns: DescribeAddonResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeAddonVersions")
    def describe_addon_versions(
        self,
        context: RequestContext,
        kubernetes_version: String = None,
        max_results: DescribeAddonVersionsRequestMaxResults = None,
        next_token: String = None,
        addon_name: String = None,
    ) -> DescribeAddonVersionsResponse:
        """Describes the Kubernetes versions that the add-on can be used with.

        :param kubernetes_version: The Kubernetes versions that the add-on can be used with.
        :param max_results: The maximum number of results to return.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``DescribeAddonVersionsRequest`` where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :param addon_name: The name of the add-on.
        :returns: DescribeAddonVersionsResponse
        :raises ServerException:
        :raises ResourceNotFoundException:
        :raises InvalidParameterException:
        """
        raise NotImplementedError

    @handler("DescribeCluster")
    def describe_cluster(self, context: RequestContext, name: String) -> DescribeClusterResponse:
        """Returns descriptive information about an Amazon EKS cluster.

        The API server endpoint and certificate authority data returned by this
        operation are required for ``kubelet`` and ``kubectl`` to communicate
        with your Kubernetes API server. For more information, see `Create a
        kubeconfig for Amazon
        EKS <https://docs.aws.amazon.com/eks/latest/userguide/create-kubeconfig.html>`__.

        The API server endpoint and certificate authority data aren't available
        until the cluster reaches the ``ACTIVE`` state.

        :param name: The name of the cluster to describe.
        :returns: DescribeClusterResponse
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DescribeFargateProfile")
    def describe_fargate_profile(
        self, context: RequestContext, cluster_name: String, fargate_profile_name: String
    ) -> DescribeFargateProfileResponse:
        """Returns descriptive information about an Fargate profile.

        :param cluster_name: The name of the Amazon EKS cluster associated with the Fargate profile.
        :param fargate_profile_name: The name of the Fargate profile to describe.
        :returns: DescribeFargateProfileResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("DescribeIdentityProviderConfig")
    def describe_identity_provider_config(
        self,
        context: RequestContext,
        cluster_name: String,
        identity_provider_config: IdentityProviderConfig,
    ) -> DescribeIdentityProviderConfigResponse:
        """Returns descriptive information about an identity provider
        configuration.

        :param cluster_name: The cluster name that the identity provider configuration is associated
        to.
        :param identity_provider_config: An object that represents an identity provider configuration.
        :returns: DescribeIdentityProviderConfigResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DescribeNodegroup")
    def describe_nodegroup(
        self, context: RequestContext, cluster_name: String, nodegroup_name: String
    ) -> DescribeNodegroupResponse:
        """Returns descriptive information about an Amazon EKS node group.

        :param cluster_name: The name of the Amazon EKS cluster associated with the node group.
        :param nodegroup_name: The name of the node group to describe.
        :returns: DescribeNodegroupResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("DescribeUpdate")
    def describe_update(
        self,
        context: RequestContext,
        name: String,
        update_id: String,
        nodegroup_name: String = None,
        addon_name: String = None,
    ) -> DescribeUpdateResponse:
        """Returns descriptive information about an update against your Amazon EKS
        cluster or associated managed node group or Amazon EKS add-on.

        When the status of the update is ``Succeeded``, the update is complete.
        If an update fails, the status is ``Failed``, and an error detail
        explains the reason for the failure.

        :param name: The name of the Amazon EKS cluster associated with the update.
        :param update_id: The ID of the update to describe.
        :param nodegroup_name: The name of the Amazon EKS node group associated with the update.
        :param addon_name: The name of the add-on.
        :returns: DescribeUpdateResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("DisassociateIdentityProviderConfig")
    def disassociate_identity_provider_config(
        self,
        context: RequestContext,
        cluster_name: String,
        identity_provider_config: IdentityProviderConfig,
        client_request_token: String = None,
    ) -> DisassociateIdentityProviderConfigResponse:
        """Disassociates an identity provider configuration from a cluster. If you
        disassociate an identity provider from your cluster, users included in
        the provider can no longer access the cluster. However, you can still
        access the cluster with Amazon Web Services IAM users.

        :param cluster_name: The name of the cluster to disassociate an identity provider from.
        :param identity_provider_config: An object that represents an identity provider configuration.
        :param client_request_token: A unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: DisassociateIdentityProviderConfigResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("ListAddons")
    def list_addons(
        self,
        context: RequestContext,
        cluster_name: ClusterName,
        max_results: ListAddonsRequestMaxResults = None,
        next_token: String = None,
    ) -> ListAddonsResponse:
        """Lists the available add-ons.

        :param cluster_name: The name of the cluster.
        :param max_results: The maximum number of add-on results returned by ``ListAddonsRequest``
        in paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``ListAddonsRequest`` where ``maxResults`` was used and the results
        exceeded the value of that parameter.
        :returns: ListAddonsResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ClientException:
        :raises ResourceNotFoundException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("ListClusters")
    def list_clusters(
        self,
        context: RequestContext,
        max_results: ListClustersRequestMaxResults = None,
        next_token: String = None,
        include: IncludeClustersList = None,
    ) -> ListClustersResponse:
        """Lists the Amazon EKS clusters in your Amazon Web Services account in the
        specified Region.

        :param max_results: The maximum number of cluster results returned by ``ListClusters`` in
        paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``ListClusters`` request where ``maxResults`` was used and the results
        exceeded the value of that parameter.
        :param include: Indicates whether external clusters are included in the returned list.
        :returns: ListClustersResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("ListFargateProfiles")
    def list_fargate_profiles(
        self,
        context: RequestContext,
        cluster_name: String,
        max_results: FargateProfilesRequestMaxResults = None,
        next_token: String = None,
    ) -> ListFargateProfilesResponse:
        """Lists the Fargate profiles associated with the specified cluster in your
        Amazon Web Services account in the specified Region.

        :param cluster_name: The name of the Amazon EKS cluster that you would like to list Fargate
        profiles in.
        :param max_results: The maximum number of Fargate profile results returned by
        ``ListFargateProfiles`` in paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``ListFargateProfiles`` request where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :returns: ListFargateProfilesResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("ListIdentityProviderConfigs")
    def list_identity_provider_configs(
        self,
        context: RequestContext,
        cluster_name: String,
        max_results: ListIdentityProviderConfigsRequestMaxResults = None,
        next_token: String = None,
    ) -> ListIdentityProviderConfigsResponse:
        """A list of identity provider configurations.

        :param cluster_name: The cluster name that you want to list identity provider configurations
        for.
        :param max_results: The maximum number of identity provider configurations returned by
        ``ListIdentityProviderConfigs`` in paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``IdentityProviderConfigsRequest`` where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :returns: ListIdentityProviderConfigsResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("ListNodegroups")
    def list_nodegroups(
        self,
        context: RequestContext,
        cluster_name: String,
        max_results: ListNodegroupsRequestMaxResults = None,
        next_token: String = None,
    ) -> ListNodegroupsResponse:
        """Lists the Amazon EKS managed node groups associated with the specified
        cluster in your Amazon Web Services account in the specified Region.
        Self-managed node groups are not listed.

        :param cluster_name: The name of the Amazon EKS cluster that you would like to list node
        groups in.
        :param max_results: The maximum number of node group results returned by ``ListNodegroups``
        in paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``ListNodegroups`` request where ``maxResults`` was used and the results
        exceeded the value of that parameter.
        :returns: ListNodegroupsResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: String
    ) -> ListTagsForResourceResponse:
        """List the tags for an Amazon EKS resource.

        :param resource_arn: The Amazon Resource Name (ARN) that identifies the resource for which to
        list the tags.
        :returns: ListTagsForResourceResponse
        :raises BadRequestException:
        :raises NotFoundException:
        """
        raise NotImplementedError

    @handler("ListUpdates")
    def list_updates(
        self,
        context: RequestContext,
        name: String,
        nodegroup_name: String = None,
        addon_name: String = None,
        next_token: String = None,
        max_results: ListUpdatesRequestMaxResults = None,
    ) -> ListUpdatesResponse:
        """Lists the updates associated with an Amazon EKS cluster or managed node
        group in your Amazon Web Services account, in the specified Region.

        :param name: The name of the Amazon EKS cluster to list updates for.
        :param nodegroup_name: The name of the Amazon EKS managed node group to list updates for.
        :param addon_name: The names of the installed add-ons that have available updates.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``ListUpdates`` request where ``maxResults`` was used and the results
        exceeded the value of that parameter.
        :param max_results: The maximum number of update results returned by ``ListUpdates`` in
        paginated output.
        :returns: ListUpdatesResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("RegisterCluster")
    def register_cluster(
        self,
        context: RequestContext,
        name: ClusterName,
        connector_config: ConnectorConfigRequest,
        client_request_token: String = None,
        tags: TagMap = None,
    ) -> RegisterClusterResponse:
        """Connects a Kubernetes cluster to the Amazon EKS control plane.

        Any Kubernetes cluster can be connected to the Amazon EKS control plane
        to view current information about the cluster and its nodes.

        Cluster connection requires two steps. First, send a
        ``RegisterClusterRequest`` to add it to the Amazon EKS control plane.

        Second, a
        `Manifest <https://amazon-eks.s3.us-west-2.amazonaws.com/eks-connector/manifests/eks-connector/latest/eks-connector.yaml>`__
        containing the ``activationID`` and ``activationCode`` must be applied
        to the Kubernetes cluster through it's native provider to provide
        visibility.

        After the Manifest is updated and applied, then the connected cluster is
        visible to the Amazon EKS control plane. If the Manifest is not applied
        within three days, then the connected cluster will no longer be visible
        and must be deregistered. See DeregisterCluster.

        :param name: Define a unique name for this cluster for your Region.
        :param connector_config: The configuration settings required to connect the Kubernetes cluster to
        the Amazon EKS control plane.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param tags: The metadata that you apply to the cluster to assist with categorization
        and organization.
        :returns: RegisterClusterResponse
        :raises ResourceLimitExceededException:
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ServiceUnavailableException:
        :raises AccessDeniedException:
        :raises ResourceInUseException:
        :raises ResourcePropagationDelayException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: String, tags: TagMap
    ) -> TagResourceResponse:
        """Associates the specified tags to a resource with the specified
        ``resourceArn``. If existing tags on a resource are not specified in the
        request parameters, they are not changed. When a resource is deleted,
        the tags associated with that resource are deleted as well. Tags that
        you create for Amazon EKS resources do not propagate to any other
        resources associated with the cluster. For example, if you tag a cluster
        with this operation, that tag does not automatically propagate to the
        subnets and nodes associated with the cluster.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource to which to add tags.
        :param tags: The tags to add to the resource.
        :returns: TagResourceResponse
        :raises BadRequestException:
        :raises NotFoundException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: String, tag_keys: TagKeyList
    ) -> UntagResourceResponse:
        """Deletes specified tags from a resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource from which to delete
        tags.
        :param tag_keys: The keys of the tags to be removed.
        :returns: UntagResourceResponse
        :raises BadRequestException:
        :raises NotFoundException:
        """
        raise NotImplementedError

    @handler("UpdateAddon")
    def update_addon(
        self,
        context: RequestContext,
        cluster_name: ClusterName,
        addon_name: String,
        addon_version: String = None,
        service_account_role_arn: RoleArn = None,
        resolve_conflicts: ResolveConflicts = None,
        client_request_token: String = None,
    ) -> UpdateAddonResponse:
        """Updates an Amazon EKS add-on.

        :param cluster_name: The name of the cluster.
        :param addon_name: The name of the add-on.
        :param addon_version: The version of the add-on.
        :param service_account_role_arn: The Amazon Resource Name (ARN) of an existing IAM role to bind to the
        add-on's service account.
        :param resolve_conflicts: How to resolve parameter value conflicts when applying the new version
        of the add-on to the cluster.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: UpdateAddonResponse
        :raises InvalidParameterException:
        :raises InvalidRequestException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("UpdateClusterConfig")
    def update_cluster_config(
        self,
        context: RequestContext,
        name: String,
        resources_vpc_config: VpcConfigRequest = None,
        logging: Logging = None,
        client_request_token: String = None,
    ) -> UpdateClusterConfigResponse:
        """Updates an Amazon EKS cluster configuration. Your cluster continues to
        function during the update. The response output includes an update ID
        that you can use to track the status of your cluster update with the
        DescribeUpdate API operation.

        You can use this API operation to enable or disable exporting the
        Kubernetes control plane logs for your cluster to CloudWatch Logs. By
        default, cluster control plane logs aren't exported to CloudWatch Logs.
        For more information, see `Amazon EKS Cluster Control Plane
        Logs <https://docs.aws.amazon.com/eks/latest/userguide/control-plane-logs.html>`__
        in the *Amazon EKS User Guide* .

        CloudWatch Logs ingestion, archive storage, and data scanning rates
        apply to exported control plane logs. For more information, see
        `CloudWatch Pricing <http://aws.amazon.com/cloudwatch/pricing/>`__.

        You can also use this API operation to enable or disable public and
        private access to your cluster's Kubernetes API server endpoint. By
        default, public access is enabled, and private access is disabled. For
        more information, see `Amazon EKS cluster endpoint access
        control <https://docs.aws.amazon.com/eks/latest/userguide/cluster-endpoint.html>`__
        in the *Amazon EKS User Guide* .

        You can't update the subnets or security group IDs for an existing
        cluster.

        Cluster updates are asynchronous, and they should finish within a few
        minutes. During an update, the cluster status moves to ``UPDATING``
        (this status transition is eventually consistent). When the update is
        complete (either ``Failed`` or ``Successful``), the cluster status moves
        to ``Active``.

        :param name: The name of the Amazon EKS cluster to update.
        :param resources_vpc_config: An object representing the VPC configuration to use for an Amazon EKS
        cluster.
        :param logging: Enable or disable exporting the Kubernetes control plane logs for your
        cluster to CloudWatch Logs.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: UpdateClusterConfigResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("UpdateClusterVersion")
    def update_cluster_version(
        self,
        context: RequestContext,
        name: String,
        version: String,
        client_request_token: String = None,
    ) -> UpdateClusterVersionResponse:
        """Updates an Amazon EKS cluster to the specified Kubernetes version. Your
        cluster continues to function during the update. The response output
        includes an update ID that you can use to track the status of your
        cluster update with the DescribeUpdate API operation.

        Cluster updates are asynchronous, and they should finish within a few
        minutes. During an update, the cluster status moves to ``UPDATING``
        (this status transition is eventually consistent). When the update is
        complete (either ``Failed`` or ``Successful``), the cluster status moves
        to ``Active``.

        If your cluster has managed node groups attached to it, all of your node
        groups’ Kubernetes versions must match the cluster’s Kubernetes version
        in order to update the cluster to a new Kubernetes version.

        :param name: The name of the Amazon EKS cluster to update.
        :param version: The desired Kubernetes version following a successful update.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: UpdateClusterVersionResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("UpdateNodegroupConfig")
    def update_nodegroup_config(
        self,
        context: RequestContext,
        cluster_name: String,
        nodegroup_name: String,
        labels: UpdateLabelsPayload = None,
        taints: UpdateTaintsPayload = None,
        scaling_config: NodegroupScalingConfig = None,
        update_config: NodegroupUpdateConfig = None,
        client_request_token: String = None,
    ) -> UpdateNodegroupConfigResponse:
        """Updates an Amazon EKS managed node group configuration. Your node group
        continues to function during the update. The response output includes an
        update ID that you can use to track the status of your node group update
        with the DescribeUpdate API operation. Currently you can update the
        Kubernetes labels for a node group or the scaling configuration.

        :param cluster_name: The name of the Amazon EKS cluster that the managed node group resides
        in.
        :param nodegroup_name: The name of the managed node group to update.
        :param labels: The Kubernetes labels to be applied to the nodes in the node group after
        the update.
        :param taints: The Kubernetes taints to be applied to the nodes in the node group after
        the update.
        :param scaling_config: The scaling configuration details for the Auto Scaling group after the
        update.
        :param update_config: The node group update configuration.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: UpdateNodegroupConfigResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError

    @handler("UpdateNodegroupVersion")
    def update_nodegroup_version(
        self,
        context: RequestContext,
        cluster_name: String,
        nodegroup_name: String,
        version: String = None,
        release_version: String = None,
        launch_template: LaunchTemplateSpecification = None,
        force: Boolean = None,
        client_request_token: String = None,
    ) -> UpdateNodegroupVersionResponse:
        """Updates the Kubernetes version or AMI version of an Amazon EKS managed
        node group.

        You can update a node group using a launch template only if the node
        group was originally deployed with a launch template. If you need to
        update a custom AMI in a node group that was deployed with a launch
        template, then update your custom AMI, specify the new ID in a new
        version of the launch template, and then update the node group to the
        new version of the launch template.

        If you update without a launch template, then you can update to the
        latest available AMI version of a node group's current Kubernetes
        version by not specifying a Kubernetes version in the request. You can
        update to the latest AMI version of your cluster's current Kubernetes
        version by specifying your cluster's Kubernetes version in the request.
        For more information, see `Amazon EKS optimized Amazon Linux 2 AMI
        versions <https://docs.aws.amazon.com/eks/latest/userguide/eks-linux-ami-versions.html>`__
        in the *Amazon EKS User Guide*.

        You cannot roll back a node group to an earlier Kubernetes version or
        AMI version.

        When a node in a managed node group is terminated due to a scaling
        action or update, the pods in that node are drained first. Amazon EKS
        attempts to drain the nodes gracefully and will fail if it is unable to
        do so. You can ``force`` the update if Amazon EKS is unable to drain the
        nodes as a result of a pod disruption budget issue.

        :param cluster_name: The name of the Amazon EKS cluster that is associated with the managed
        node group to update.
        :param nodegroup_name: The name of the managed node group to update.
        :param version: The Kubernetes version to update to.
        :param release_version: The AMI version of the Amazon EKS optimized AMI to use for the update.
        :param launch_template: An object representing a node group's launch template specification.
        :param force: Force the update if the existing node group's pods are unable to be
        drained due to a pod disruption budget issue.
        :param client_request_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :returns: UpdateNodegroupVersionResponse
        :raises InvalidParameterException:
        :raises ClientException:
        :raises ServerException:
        :raises ResourceInUseException:
        :raises ResourceNotFoundException:
        :raises InvalidRequestException:
        """
        raise NotImplementedError



#!/usr/bin/env python
_G='uname -a'
_F='Unsupported operation "%s"'
_E='Terminating local daemon process (port %s)'
_D='result'
_C='Content-Length'
_B='error'
_A=True
import json,logging,os,shutil,socket,struct,subprocess,sys,tempfile,threading,traceback,uuid
from http.server import BaseHTTPRequestHandler,HTTPServer
from socketserver import ThreadingMixIn
import boto3,requests
from localstack.utils.common import is_port_open
LOG=logging.getLogger('local_daemon')
DEFAULT_PORT_LOCAL_DAEMON=4600
DEFAULT_PORT_LOCAL_DAEMON_ROOT=4601
DEFAULT_PORT_S3=4566
DEFAULT_PORT_EC2=4566
ENDPOINT_S3=f"http://localhost:{DEFAULT_PORT_S3}"
ENDPOINT_EC2=f"http://localhost:{DEFAULT_PORT_EC2}"
LOCAL_BIND_ADDRESS_PATTERN='192.168.123.*'
USED_BIND_ADDRESSES=set()
MAC_NETWORK_INTERFACE='en0'
BUCKET_MARKER_LOCAL=(os.environ.get('BUCKET_MARKER_LOCAL')or'').strip()or'__local__'
class FuncThread(threading.Thread):
	def __init__(A,func,params=None):threading.Thread.__init__(A);A.daemon=_A;A.params=params;A.func=func
	def run(A):
		try:A.func(A.params)
		except Exception as B:log('Error in thread function: %s %s'%(B,traceback.format_exc()))
class ThreadedHTTPServer(ThreadingMixIn,HTTPServer):daemon_threads=_A
class RequestHandler(BaseHTTPRequestHandler):
	def do_POST(A):
		A.read_content()
		try:B=A.handle_request();A.send_response(200)
		except Exception as C:D=str(C);B=json.dumps({_B:D});log('Error handling request: %s - %s'%(A.request_json,C));A.send_response(500)
		A.send_header(_C,'%s'%len(B)if B else 0);A.end_headers()
		if len(B or''):A.wfile.write(to_bytes(B))
	def handle_request(D):
		C=D.request_json;A='{}';B=C.get('op','')
		if B=='getos':A={_D:get_os()}
		elif B=='shell':E=C.get('command');A=run_shell_cmd(E)
		elif B=='s3:download':A=s3_download(C)
		elif B.startswith('root:'):A=forward_root_request(C)
		elif B=='kill':log(_E%DEFAULT_PORT_LOCAL_DAEMON);os._exit(0)
		else:A={_B:_F%B}
		A=json.dumps(A)if isinstance(A,dict)else A;return A
	def read_content(A):
		if hasattr(A,'data_bytes'):return
		B=A.headers.get(_C);A.data_bytes=A.rfile.read(int(B));A.request_json={}
		try:A.request_json=json.loads(A.data_bytes)
		except Exception:pass
class RequestHandlerRoot(RequestHandler):
	def handle_request(D):
		C=D.request_json;A='{}';B=C.get('op')
		if B=='root:ssh_proxy':A=start_ssh_forward_proxy(C)
		elif B=='kill':log(_E%DEFAULT_PORT_LOCAL_DAEMON_ROOT);os._exit(0)
		else:A={_B:_F%B}
		A=json.dumps(A)if isinstance(A,dict)else A;return A
def s3_download(request):
	B=request;C=B['bucket'];D=B['key'];E=os.environ.get('TMPDIR')or tempfile.mkdtemp();A=os.path.join(E,B.get('file_name')or's3file.%s'%str(uuid.uuid4()))
	if not os.path.exists(A)or B.get('overwrite'):
		if C==BUCKET_MARKER_LOCAL:shutil.copy(D,A)
		else:F=boto3.client('s3',endpoint_url=ENDPOINT_S3);log('Downloading S3 file s3://%s/%s to %s'%(C,D,A));F.download_file(C,D,A)
	return{'local_file':A}
def forward_root_request(request):A=f"http://localhost:{DEFAULT_PORT_LOCAL_DAEMON_ROOT}";B=requests.post(A,data=json.dumps(request));return json.loads(to_str(B.content))
def start_ssh_forward_proxy(options):E='port';A=options;from localstack_ext.bootstrap.tcp_proxy import server_loop as F;B=22;C=A.get(E)or get_free_tcp_port();D=next_available_bind_address(B);log(f"Starting local SSH forward proxy, {D}:{B} -> localhost:{C}");A={'bind_port':B,'bind_addr':D,E:C};FuncThread(F,A).start();return{'host':D,'forward_port':C}
def next_available_bind_address(port):
	B=port;C=len(USED_BIND_ADDRESSES)+2
	for E in range(C,C+30):
		A=LOCAL_BIND_ADDRESS_PATTERN.replace('*',str(E));create_network_interface_alias(A)
		if is_port_open(f"tcp://{A}:{B}"):continue
		D=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
		with D:
			try:D.bind((A,B))
			except Exception:continue
		USED_BIND_ADDRESSES.add(A);return A
	raise Exception(f"Unable to determine free bind address for port {B}")
def create_network_interface_alias(address,interface=None):
	B=address;A=interface;E='sudo'
	if is_mac_os():A=A or MAC_NETWORK_INTERFACE;run_cmd(f"{E} ifconfig {A} alias {B}");return
	if is_linux():
		C=os.listdir('/sys/class/net/');C=[A for A in C if':'not in A]
		for A in C:
			try:D=get_ip_address(A);log(f"Found network interface {A} with address {D}");assert D;assert A not in['lo']and not D.startswith('127.');run_cmd(f"{E} ifconfig {A}:0 {B} netmask 255.255.255.0 up");return
			except Exception as F:log(f"Unable to create forward proxy on interface {A}, address {B}: {F}")
	raise Exception('Unable to create network interface')
def run_shell_cmd(command):
	try:return{_D:run_cmd(command)}
	except Exception as A:
		B=str(A)
		if isinstance(A,subprocess.CalledProcessError):B='%s: %s'%(B,A.output)
		return{_B:B}
def get_os():
	if is_mac_os():return'macos'
	if is_linux():return'linux'
	return'windows'
def run_cmd(cmd):log(f"Running command: {cmd}");return to_str(subprocess.check_output(cmd,stderr=subprocess.STDOUT,shell=_A))
def log(*A):print(*(A));sys.stdout.flush()
def is_mac_os():
	try:A=to_str(subprocess.check_output(_G,shell=_A));return'Darwin'in A
	except subprocess.CalledProcessError:return False
def is_linux():
	try:A=to_str(subprocess.check_output(_G,shell=_A));return'Linux'in A
	except subprocess.CalledProcessError:return False
def get_ip_address(ifname):import fcntl;A=socket.socket(socket.AF_INET,socket.SOCK_DGRAM);return socket.inet_ntoa(fcntl.ioctl(A.fileno(),35093,struct.pack('256s',to_bytes(ifname[:15])))[20:24])
def get_free_tcp_port():A=socket.socket(socket.AF_INET,socket.SOCK_STREAM);A.bind(('',0));C,B=A.getsockname();A.close();return B
def to_bytes(obj):A=obj;return A.encode('utf-8')if isinstance(A,str)else A
def to_str(obj):A=obj;return A.decode('utf-8')if isinstance(A,bytes)else A
def start_server(port,handler):
	A=port;kill_server(A)
	try:log(f"Starting local daemon server on port {A}");B=ThreadedHTTPServer(('0.0.0.0',A),handler);B.serve_forever()
	except Exception:log(f"Local daemon server already running, or port {A} not available")
def kill_server(port):
	try:requests.post(f"http://localhost:{port}",data='{"op":"kill"}')
	except Exception:pass
def kill_servers():kill_server(DEFAULT_PORT_LOCAL_DAEMON);kill_server(DEFAULT_PORT_LOCAL_DAEMON_ROOT)
def main():
	E='AWS_SECRET_ACCESS_KEY';D='test';C='AWS_ACCESS_KEY_ID';B='main';logging.basicConfig();A=sys.argv[1]if len(sys.argv)>1 else B;os.environ[C]=os.environ.get(C)or D;os.environ[E]=os.environ.get(E)or D
	if A==B:start_server(DEFAULT_PORT_LOCAL_DAEMON,RequestHandler)
	elif A=='root':start_server(DEFAULT_PORT_LOCAL_DAEMON_ROOT,RequestHandlerRoot)
	else:log(f"Unexpected local daemon type: {A}")
def get_log_file_path():from localstack.config import dirs;return os.path.join(dirs.tmp,'localstack_daemon.log')
def start_in_background():
	from localstack.utils.common import run as B;A=get_log_file_path();LOG.info('Logging local daemon output to %s',A);E=sys.executable;C=f"{E} {__file__}";B(C,outfile=A,asynchronous=_A);LOG.info('Attempting to obtain sudo privileges for local daemon of EC2 API (required to start SSH forward proxy on privileged port 22). You may be asked for your sudo password.');B('sudo -v',stdin=_A)
	def F(*E,asynchronous=None):D=f"sudo {C} root >> {A}";B(D,outfile=A,stdin=_A,asynchronous=asynchronous)
	D=FuncThread(F);D.start();return D
if __name__=='__main__':main()


_J='Description'
_I='InstanceId'
_H='tag:ec2_vm_manager'
_G='Name'
_F='InstanceIds'
_E=False
_D='Code'
_C='docker'
_B=None
_A=True
import base64,logging,os,tempfile
from typing import Dict,List,Tuple
import parse
from cryptography.hazmat.primitives import serialization
from localstack import config
from localstack.aws.api import RequestContext
from localstack.aws.api.ec2 import CopyImageRequest,CopyImageResult,CreateImageRequest,CreateImageResult,DescribeImagesRequest,DescribeImagesResult,DescribeInstancesRequest,DescribeInstancesResult,ImageState,InstanceState,InstanceStateChange,InstanceStateName,RegisterImageRequest,RegisterImageResult,Reservation,RunInstancesRequest,StartInstancesRequest,StartInstancesResult,StopInstancesRequest,StopInstancesResult,TerminateInstancesRequest,TerminateInstancesResult
from localstack.services.install import ARTIFACTS_REPO
from localstack.services.moto import call_moto
from localstack.utils.common import get_free_tcp_port
from localstack.utils.container_utils.container_client import ContainerException,NoSuchContainer,NoSuchImage
from localstack.utils.container_utils.docker_sdk_client import PortMappings
from localstack.utils.docker_utils import DOCKER_CLIENT
from localstack.utils.files import chmod_r,file_exists_not_empty
from localstack.utils.strings import short_uid,to_str
from localstack.utils.sync import retry
from localstack.utils.threads import start_thread
from moto.ec2 import ec2_backends
from moto.ec2.models.amis import Ami
from localstack_ext.bootstrap.install import download_github_artifact
from localstack_ext.services.ec2.vmmanager.base import IncorrectInstanceStateError,InstanceStateCode,InvalidAMIIdError,InvalidInstanceIdError,MissingParameterError,VmManager
from localstack_ext.services.ec2.vmmanager.virtualbox import call_local_daemon,is_daemon_running
LOG=logging.getLogger(__name__)
URL_DROPBEAR=f"{ARTIFACTS_REPO}/raw/master/ssh-server/dropbear"
URL_SCP=f"{ARTIFACTS_REPO}/raw/master/ssh-server/scp"
CLOUD_INIT_LOG_DIR='/var/log'
CLOUD_INIT_LOG_PATH=f"{CLOUD_INIT_LOG_DIR}/cloud-init-output.log"
DOCKER_PREFIX='localstack-ec2'
CONTAINER_NAME_TEMPL='{docker_prefix}.{instance_id}'
IMAGE_NAME_TEMPL='{docker_prefix}/{ami_name}'
CONTAINER_EC2_STATE_MAPPING={'created':(InstanceStateCode.pending,InstanceStateName.pending),'running':(InstanceStateCode.running,InstanceStateName.running),'restarting':(InstanceStateCode.shutting_down,InstanceStateName.shutting_down),'exited':(InstanceStateCode.terminated,InstanceStateName.terminated),'paused':(InstanceStateCode.stopped,InstanceStateName.stopped),'dead':(InstanceStateCode.terminated,InstanceStateName.terminated)}
UBUNTU_FOCAL_AMI='ami-ff0fea8310f3','ubuntu-20.04-focal-fossa'
class DockerVmManager(VmManager):
	@staticmethod
	def impl_name():return _C
	@staticmethod
	def image_name_from_ami_name(ami_name):return IMAGE_NAME_TEMPL.format(docker_prefix=DOCKER_PREFIX,ami_name=ami_name)
	@staticmethod
	def container_name_from_instance_id(instance_id,verify=_A):
		A=instance_id;B=CONTAINER_NAME_TEMPL.format(docker_prefix=DOCKER_PREFIX,instance_id=A)
		if verify:
			try:DOCKER_CLIENT.inspect_container(B)
			except NoSuchContainer:raise InvalidInstanceIdError(A)
		return B
	@staticmethod
	def image_from_ami_id(ami_id,verify=_A):
		A=ami_id
		for B in DOCKER_CLIENT.get_docker_image_names():
			C,E,D=B.rpartition(':')
			if C.startswith(DOCKER_PREFIX)and D==A:return B
		if verify:raise InvalidAMIIdError(A)
		return''
	@staticmethod
	def ami_id_from_image(docker_image):return docker_image.split(':')[-1]
	@staticmethod
	def ami_name_from_image(docker_image):A,B=docker_image.split(':');return parse.parse(IMAGE_NAME_TEMPL,A)['ami_name']
	@staticmethod
	def instance_id_from_container_name(container_name):return parse.parse(CONTAINER_NAME_TEMPL,container_name)['instance_id']
	def instance_state(B,instance_id):
		A=instance_id;C=B.container_name_from_instance_id(A,verify=_E)
		try:D=DOCKER_CLIENT.inspect_container(C)
		except NoSuchContainer:raise InvalidInstanceIdError(A)
		E=D['State']['Status'];F,G=CONTAINER_EC2_STATE_MAPPING[E];return InstanceState(Code=F,Name=G)
	def initialise_images(B):
		C=[('ami-a33ac4f1069a','alpine-3.14.4','alpine:3.14.4'),(*(UBUNTU_FOCAL_AMI),'ubuntu:20.04')]
		for D in C:
			E,F,A=D
			try:DOCKER_CLIENT.inspect_image(A)
			except NoSuchImage:DOCKER_CLIENT.pull_image(A)
			DOCKER_CLIENT.tag_image(source_ref=A,target_name='{}:{}'.format(B.image_name_from_ami_name(F),E))
	def start_instances(B,context,start_instances_request):
		E=start_instances_request[_F];F=ec2_backends[context.region];D=[]
		for A in E:
			LOG.debug('Starting EC2 instance %s'%A);G=B.container_name_from_instance_id(A,verify=_A);C=B.instance_state(A)
			if C[_D]==InstanceStateCode.stopped:DOCKER_CLIENT.unpause_container(G)
			elif C[_D]==InstanceStateCode.terminated:raise IncorrectInstanceStateError(A)
			H=B.instance_state(A);D.append(InstanceStateChange(CurrentState=H,PreviousState=C,InstanceId=A));F.get_instance(A).start()
		return StartInstancesResult(StartingInstances=D)
	def stop_instances(B,context,stop_instances_request):
		E=stop_instances_request[_F];F=ec2_backends[context.region];C=[]
		for A in E:
			LOG.debug('Stopping EC2 instance %s'%A);G=B.container_name_from_instance_id(A,verify=_A);D=B.instance_state(A)
			if D[_D]==InstanceStateCode.running:DOCKER_CLIENT.pause_container(G)
			H=B.instance_state(A);C.append(InstanceStateChange(CurrentState=H,PreviousState=D,InstanceId=A));I=F.get_instance(A);I.stop()
		return StopInstancesResult(StoppingInstances=C)
	def terminate_instances(B,context,terminate_instances_request):
		E=terminate_instances_request[_F];F=ec2_backends[context.region];C=[]
		for A in E:
			LOG.debug('Terminating EC2 instance %s'%A);G=B.container_name_from_instance_id(A,verify=_A);D=B.instance_state(A)
			if D[_D]in(InstanceStateCode.running,InstanceStateCode.stopped):DOCKER_CLIENT.stop_container(G)
			H=B.instance_state(A);C.append(InstanceStateChange(CurrentState=H,PreviousState=D,InstanceId=A));I=F.get_instance(A);I.terminate()
		return TerminateInstancesResult(TerminatingInstances=C)
	def run_instances(H,context,run_instances_request):
		S='/var/run/docker.sock';R='ImageId';M=context;I=run_instances_request;A=I.get(R)
		if not A:raise MissingParameterError(R)
		T=I.get('UserData');B=ec2_backends[M.region]
		try:U=H.image_from_ami_id(A,verify=_A)
		except InvalidAMIIdError as J:
			if A in B.amis.keys()and B.amis[A].get_filter_value(_H)==_C:LOG.debug(f"Deregistering AMI '{A}' because it no longer exists in Docker");B.amis.pop(A,_B)
			raise J
		N=I.get('KeyName');E='';O=call_moto(M)
		if N:
			C=B.describe_key_pairs([N])[0]
			if C.material.startswith('ssh-rsa'):E=C.material
			elif C.material.startswith('-----BEGIN RSA PRIVATE KEY-----'):V=serialization.load_pem_private_key(C.material.encode(),password=_B);W=V.public_key().public_bytes(encoding=serialization.Encoding.OpenSSH,format=serialization.PublicFormat.OpenSSH);E=to_str(W)
			else:E=to_str(base64.b64decode(C.material))
		for X in O['Instances']:
			D=X[_I];F=H.container_name_from_instance_id(D,verify=_E);P=PortMappings();K='127.0.0.1';G=get_free_tcp_port();L=[f"{K}:{G}"]
			if is_daemon_running():Q=call_local_daemon({'op':'root:ssh_proxy'});K=Q.get('host');G=Q.get('forward_port');L=[f"{K}:22",f"127.0.0.1:{G}"]
			P.add(G,22)
			try:
				LOG.debug('Launching instance %s',D);DOCKER_CLIENT.run_container(U,remove=_E,env_vars={},name=F,command=['sleep','43200'],detach=_A,ports=P,mount_volumes=[(S,S)]);Y=DOCKER_CLIENT.inspect_container(F)['NetworkSettings']['IPAddress'];L.append(f"{Y}:22");LOG.info('Instance %s will be accessible via SSH at: %s',D,', '.join(L))
				def Z():assert F in DOCKER_CLIENT.get_running_container_names()
				retry(Z,sleep=1,retries=10);start_thread(H._cloud_init,(F,E,T))
			except ContainerException as J:LOG.warning('Error launching instance %s: %s',D,J);B.get_instance(D).start()
		return O
	def describe_instances(D,context,describe_instances_request):
		C=context;E=ec2_backends[C.region]
		for A in E.all_instances():
			try:B=D.instance_state(A.id)[_D]
			except InvalidInstanceIdError:continue
			if B==InstanceStateCode.stopped and A._state.code!=InstanceStateCode.stopped:A.stop()
			elif B==InstanceStateCode.terminated and A._state.code!=InstanceStateCode.terminated:A.terminate()
			elif B==InstanceStateCode.running and A._state.code!=InstanceStateCode.running:A.start()
		return call_moto(C)
	def create_image(B,context,create_image_request):C=create_image_request;D=C[_I];E=C[_G];F=B.image_name_from_ami_name(E);A=f"ami-{short_uid()}";G=B.container_name_from_instance_id(D,verify=_A);LOG.debug('Creating image %s named %s from instance %s',A,E,D);DOCKER_CLIENT.commit(G,F,A);return CreateImageResult(ImageId=A)
	def register_image(I,context,register_image_request):
		E='RootDeviceName';A=register_image_request;C=ec2_backends[context.region];B=f"ami-{short_uid()}";F=A[E];G=next((B for B in A['BlockDeviceMappings']if B['DeviceName']==F))
		if'Ebs'in G:D='ebs'
		else:D='standard'
		H=Ami(C,B,instance=_B,source_ami=_B,name=A[_G],description=A[_J],root_device_name=A[E],root_device_type=D);C.amis[B]=H;return RegisterImageResult(ImageId=B)
	def copy_image(F,context,copy_image_request):A=copy_image_request;B=ec2_backends[context.region];C=B.describe_images(ami_ids=[A['SourceImageId']])[0];D=f"ami-{short_uid()}";E=Ami(B,D,instance=_B,source_ami=C,name=A[_G],description=A[_J],root_device_name=C.root_device_name,root_device_type=C.root_device_type);B.amis[D]=E;return CopyImageResult(ImageId=D)
	def describe_images(E,context,describe_images_request):
		F=context;A=ec2_backends[F.region]
		for B in DOCKER_CLIENT.get_docker_image_names():
			if B.startswith(DOCKER_PREFIX):G=DOCKER_CLIENT.inspect_image(B);C=E.ami_id_from_image(B);A.amis[C]=Ami(A,ami_id=C,name=E.ami_name_from_image(B),architecture=G.get('Architecture'),creation_date=G.get('Created'),public=_A,state=ImageState.available,image_location=_C);A.amis[C].add_tag('ec2_vm_manager',_C)
		H=[A.split(':')[-1]for A in DOCKER_CLIENT.get_docker_image_names()if'ami-'in A];I=A.describe_images(filters={_H:[_C]})
		for D in I:
			if D.id not in H:LOG.debug(f"Deregistering AMI '{D.id}' because it no longer exists in Docker");A.amis.pop(D.id,_B)
		return call_moto(F)
	def _cloud_init(I,params):
		A,E,F=params;DOCKER_CLIENT.exec_in_container(A,f"mkdir -p {CLOUD_INIT_LOG_DIR}")
		try:
			LOG.debug('Starting ssh setup in container: %s',A);B=os.path.join(config.TMP_FOLDER,'dropbear-sshd')
			if not file_exists_not_empty(B):download_github_artifact(URL_DROPBEAR,B);chmod_r(B,511)
			C=os.path.join(config.TMP_FOLDER,'scp')
			if not file_exists_not_empty(C):download_github_artifact(URL_SCP,C);chmod_r(C,511)
			G='/usr/bin/dropbear';DOCKER_CLIENT.copy_into_container(A,C,'/usr/bin/scp');DOCKER_CLIENT.copy_into_container(A,B,G);DOCKER_CLIENT.exec_in_container(A,['mkdir','-p','/etc/dropbear'])
			if E:DOCKER_CLIENT.exec_in_container(A,['sh','-c',f'mkdir -p $HOME/.ssh; echo -n "{E}" >> $HOME/.ssh/authorized_keys'])
			DOCKER_CLIENT.exec_in_container(A,[G,'-R','-p','22']);LOG.debug('Finished ssh setup in container: %s',A)
		except Exception as J:LOG.warning('Failed ssh setup: %s',J)
		if F:LOG.debug('Starting userdata setup in container: %s',A);H=I.instance_id_from_container_name(A);LOG.debug('Copying userdata into container: %s',A);K=f"/var/lib/cloud/instances/{H}";D=f"/var/lib/cloud/instances/{H}/user-data.txt";L,M=tempfile.mkstemp();os.write(L,base64.b64decode(F));DOCKER_CLIENT.exec_in_container(A,f"mkdir -p {K}");DOCKER_CLIENT.copy_into_container(A,M,D);LOG.debug('Executing userdata in container: %s',A);DOCKER_CLIENT.exec_in_container(A,f"chmod +x {D}");DOCKER_CLIENT.exec_in_container(A,f'sh -c "{D} 2>&1 | tee --append {CLOUD_INIT_LOG_PATH}"');LOG.debug('Finished userdata setup in container: %s',A)


import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

Arn = str
ConfigurationProfileType = str
DeploymentStrategyId = str
Description = str
Float = float
GrowthFactor = float
Id = str
Integer = int
MaxResults = int
MinutesBetween0And24Hours = int
Name = str
NextToken = str
Percentage = float
RoleArn = str
String = str
StringWithLengthBetween0And32768 = str
StringWithLengthBetween1And2048 = str
StringWithLengthBetween1And255 = str
StringWithLengthBetween1And64 = str
TagKey = str
TagValue = str
Uri = str
Version = str


class BadRequestReason(str):
    InvalidConfiguration = "InvalidConfiguration"


class BytesMeasure(str):
    KILOBYTES = "KILOBYTES"


class DeploymentEventType(str):
    PERCENTAGE_UPDATED = "PERCENTAGE_UPDATED"
    ROLLBACK_STARTED = "ROLLBACK_STARTED"
    ROLLBACK_COMPLETED = "ROLLBACK_COMPLETED"
    BAKE_TIME_STARTED = "BAKE_TIME_STARTED"
    DEPLOYMENT_STARTED = "DEPLOYMENT_STARTED"
    DEPLOYMENT_COMPLETED = "DEPLOYMENT_COMPLETED"


class DeploymentState(str):
    BAKING = "BAKING"
    VALIDATING = "VALIDATING"
    DEPLOYING = "DEPLOYING"
    COMPLETE = "COMPLETE"
    ROLLING_BACK = "ROLLING_BACK"
    ROLLED_BACK = "ROLLED_BACK"


class EnvironmentState(str):
    READY_FOR_DEPLOYMENT = "READY_FOR_DEPLOYMENT"
    DEPLOYING = "DEPLOYING"
    ROLLING_BACK = "ROLLING_BACK"
    ROLLED_BACK = "ROLLED_BACK"


class GrowthType(str):
    LINEAR = "LINEAR"
    EXPONENTIAL = "EXPONENTIAL"


class ReplicateTo(str):
    NONE = "NONE"
    SSM_DOCUMENT = "SSM_DOCUMENT"


class TriggeredBy(str):
    USER = "USER"
    APPCONFIG = "APPCONFIG"
    CLOUDWATCH_ALARM = "CLOUDWATCH_ALARM"
    INTERNAL_ERROR = "INTERNAL_ERROR"


class ValidatorType(str):
    JSON_SCHEMA = "JSON_SCHEMA"
    LAMBDA = "LAMBDA"


class InvalidConfigurationDetail(TypedDict, total=False):
    """Detailed information about the bad request exception error when creating
    a hosted configuration version.
    """

    Constraint: Optional[String]
    Location: Optional[String]
    Reason: Optional[String]
    Type: Optional[String]


InvalidConfigurationDetailList = List[InvalidConfigurationDetail]


class BadRequestDetails(TypedDict, total=False):
    """Detailed information about the input that failed to satisfy the
    constraints specified by a call.
    """

    InvalidConfiguration: Optional[InvalidConfigurationDetailList]


class BadRequestException(ServiceException):
    """The input fails to satisfy the constraints specified by an Amazon Web
    Services service.
    """

    Message: Optional[String]
    Reason: Optional[BadRequestReason]
    Details: Optional[BadRequestDetails]


class ConflictException(ServiceException):
    """The request could not be processed because of conflict in the current
    state of the resource.
    """

    Message: Optional[String]


class InternalServerException(ServiceException):
    """There was an internal failure in the AppConfig service."""

    Message: Optional[String]


class PayloadTooLargeException(ServiceException):
    """The configuration size is too large."""

    Message: Optional[String]
    Measure: Optional[BytesMeasure]
    Limit: Optional[Float]
    Size: Optional[Float]


class ResourceNotFoundException(ServiceException):
    """The requested resource could not be found."""

    Message: Optional[String]
    ResourceName: Optional[String]


class ServiceQuotaExceededException(ServiceException):
    """The number of hosted configuration versions exceeds the limit for the
    AppConfig hosted configuration store. Delete one or more versions and
    try again.
    """

    Message: Optional[String]


class Application(TypedDict, total=False):
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]


ApplicationList = List[Application]


class Applications(TypedDict, total=False):
    Items: Optional[ApplicationList]
    NextToken: Optional[NextToken]


Blob = bytes


class Configuration(TypedDict, total=False):
    Content: Optional[Blob]
    ConfigurationVersion: Optional[Version]
    ContentType: Optional[String]


class Validator(TypedDict, total=False):
    """A validator provides a syntactic or semantic check to ensure the
    configuration that you want to deploy functions as intended. To validate
    your application configuration data, you provide a schema or an Amazon
    Web Services Lambda function that runs against the configuration. The
    configuration deployment or update can only proceed when the
    configuration data is valid.
    """

    Type: ValidatorType
    Content: StringWithLengthBetween0And32768


ValidatorList = List[Validator]


class ConfigurationProfile(TypedDict, total=False):
    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]
    LocationUri: Optional[Uri]
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]
    Type: Optional[ConfigurationProfileType]


ValidatorTypeList = List[ValidatorType]


class ConfigurationProfileSummary(TypedDict, total=False):
    """A summary of a configuration profile."""

    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[Name]
    LocationUri: Optional[Uri]
    ValidatorTypes: Optional[ValidatorTypeList]
    Type: Optional[ConfigurationProfileType]


ConfigurationProfileSummaryList = List[ConfigurationProfileSummary]


class ConfigurationProfiles(TypedDict, total=False):
    Items: Optional[ConfigurationProfileSummaryList]
    NextToken: Optional[NextToken]


TagMap = Dict[TagKey, TagValue]


class CreateApplicationRequest(ServiceRequest):
    Name: Name
    Description: Optional[Description]
    Tags: Optional[TagMap]


class CreateConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    Name: Name
    Description: Optional[Description]
    LocationUri: Uri
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]
    Tags: Optional[TagMap]
    Type: Optional[ConfigurationProfileType]


class CreateDeploymentStrategyRequest(ServiceRequest):
    Name: Name
    Description: Optional[Description]
    DeploymentDurationInMinutes: MinutesBetween0And24Hours
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthFactor: GrowthFactor
    GrowthType: Optional[GrowthType]
    ReplicateTo: ReplicateTo
    Tags: Optional[TagMap]


class Monitor(TypedDict, total=False):
    """Amazon CloudWatch alarms to monitor during the deployment process."""

    AlarmArn: StringWithLengthBetween1And2048
    AlarmRoleArn: Optional[RoleArn]


MonitorList = List[Monitor]


class CreateEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    Name: Name
    Description: Optional[Description]
    Monitors: Optional[MonitorList]
    Tags: Optional[TagMap]


class CreateHostedConfigurationVersionRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    Description: Optional[Description]
    Content: Blob
    ContentType: StringWithLengthBetween1And255
    LatestVersionNumber: Optional[Integer]


class DeleteApplicationRequest(ServiceRequest):
    ApplicationId: Id


class DeleteConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id


class DeleteDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId


class DeleteEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id


class DeleteHostedConfigurationVersionRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    VersionNumber: Integer


Iso8601DateTime = datetime


class DeploymentEvent(TypedDict, total=False):
    """An object that describes a deployment event."""

    EventType: Optional[DeploymentEventType]
    TriggeredBy: Optional[TriggeredBy]
    Description: Optional[Description]
    OccurredAt: Optional[Iso8601DateTime]


DeploymentEvents = List[DeploymentEvent]


class Deployment(TypedDict, total=False):
    ApplicationId: Optional[Id]
    EnvironmentId: Optional[Id]
    DeploymentStrategyId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    DeploymentNumber: Optional[Integer]
    ConfigurationName: Optional[Name]
    ConfigurationLocationUri: Optional[Uri]
    ConfigurationVersion: Optional[Version]
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    State: Optional[DeploymentState]
    EventLog: Optional[DeploymentEvents]
    PercentageComplete: Optional[Percentage]
    StartedAt: Optional[Iso8601DateTime]
    CompletedAt: Optional[Iso8601DateTime]


class DeploymentSummary(TypedDict, total=False):
    """Information about the deployment."""

    DeploymentNumber: Optional[Integer]
    ConfigurationName: Optional[Name]
    ConfigurationVersion: Optional[Version]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    State: Optional[DeploymentState]
    PercentageComplete: Optional[Percentage]
    StartedAt: Optional[Iso8601DateTime]
    CompletedAt: Optional[Iso8601DateTime]


DeploymentList = List[DeploymentSummary]


class DeploymentStrategy(TypedDict, total=False):
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    ReplicateTo: Optional[ReplicateTo]


DeploymentStrategyList = List[DeploymentStrategy]


class DeploymentStrategies(TypedDict, total=False):
    Items: Optional[DeploymentStrategyList]
    NextToken: Optional[NextToken]


class Deployments(TypedDict, total=False):
    Items: Optional[DeploymentList]
    NextToken: Optional[NextToken]


class Environment(TypedDict, total=False):
    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]
    State: Optional[EnvironmentState]
    Monitors: Optional[MonitorList]


EnvironmentList = List[Environment]


class Environments(TypedDict, total=False):
    Items: Optional[EnvironmentList]
    NextToken: Optional[NextToken]


class GetApplicationRequest(ServiceRequest):
    ApplicationId: Id


class GetConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id


class GetConfigurationRequest(ServiceRequest):
    Application: StringWithLengthBetween1And64
    Environment: StringWithLengthBetween1And64
    Configuration: StringWithLengthBetween1And64
    ClientId: StringWithLengthBetween1And64
    ClientConfigurationVersion: Optional[Version]


class GetDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentNumber: Integer


class GetDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId


class GetEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id


class GetHostedConfigurationVersionRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    VersionNumber: Integer


class HostedConfigurationVersion(TypedDict, total=False):
    ApplicationId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    VersionNumber: Optional[Integer]
    Description: Optional[Description]
    Content: Optional[Blob]
    ContentType: Optional[StringWithLengthBetween1And255]


class HostedConfigurationVersionSummary(TypedDict, total=False):
    """Information about the configuration."""

    ApplicationId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    VersionNumber: Optional[Integer]
    Description: Optional[Description]
    ContentType: Optional[StringWithLengthBetween1And255]


HostedConfigurationVersionSummaryList = List[HostedConfigurationVersionSummary]


class HostedConfigurationVersions(TypedDict, total=False):
    Items: Optional[HostedConfigurationVersionSummaryList]
    NextToken: Optional[NextToken]


class ListApplicationsRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListConfigurationProfilesRequest(ServiceRequest):
    ApplicationId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]
    Type: Optional[ConfigurationProfileType]


class ListDeploymentStrategiesRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListDeploymentsRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListEnvironmentsRequest(ServiceRequest):
    ApplicationId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListHostedConfigurationVersionsRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListTagsForResourceRequest(ServiceRequest):
    ResourceArn: Arn


class ResourceTags(TypedDict, total=False):
    Tags: Optional[TagMap]


class StartDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentStrategyId: DeploymentStrategyId
    ConfigurationProfileId: Id
    ConfigurationVersion: Version
    Description: Optional[Description]
    Tags: Optional[TagMap]


class StopDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentNumber: Integer


TagKeyList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    ResourceArn: Arn
    Tags: TagMap


class UntagResourceRequest(ServiceRequest):
    ResourceArn: Arn
    TagKeys: TagKeyList


class UpdateApplicationRequest(ServiceRequest):
    ApplicationId: Id
    Name: Optional[Name]
    Description: Optional[Description]


class UpdateConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    Name: Optional[Name]
    Description: Optional[Description]
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]


class UpdateDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthFactor: Optional[GrowthFactor]
    GrowthType: Optional[GrowthType]


class UpdateEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    Name: Optional[Name]
    Description: Optional[Description]
    Monitors: Optional[MonitorList]


class ValidateConfigurationRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    ConfigurationVersion: Version


class AppconfigApi:

    service = "appconfig"
    version = "2019-10-09"

    @handler("CreateApplication")
    def create_application(
        self,
        context: RequestContext,
        name: Name,
        description: Description = None,
        tags: TagMap = None,
    ) -> Application:
        """Creates an application. An application in AppConfig is a logical unit of
        code that provides capabilities for your customers. For example, an
        application can be a microservice that runs on Amazon EC2 instances, a
        mobile application installed by your users, a serverless application
        using Amazon API Gateway and Lambda, or any system you run on behalf of
        others.

        :param name: A name for the application.
        :param description: A description of the application.
        :param tags: Metadata to assign to the application.
        :returns: Application
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("CreateConfigurationProfile", expand=False)
    def create_configuration_profile(
        self, context: RequestContext, request: CreateConfigurationProfileRequest
    ) -> ConfigurationProfile:
        """Creates a configuration profile, which is information that enables
        AppConfig to access the configuration source. Valid configuration
        sources include the AppConfig hosted configuration store, Amazon Web
        Services Systems Manager (SSM) documents, SSM Parameter Store
        parameters, Amazon S3 objects, or any `integration source
        action <http://docs.aws.amazon.com/codepipeline/latest/userguide/integrations-action-type.html#integrations-source>`__
        supported by CodePipeline. A configuration profile includes the
        following information:

        -  The URI location of the configuration data.

        -  The Identity and Access Management (IAM) role that provides access to
           the configuration data.

        -  A validator for the configuration data. Available validators include
           either a JSON Schema or an Amazon Web Services Lambda function.

        For more information, see `Create a Configuration and a Configuration
        Profile <http://docs.aws.amazon.com/appconfig/latest/userguide/appconfig-creating-configuration-and-profile.html>`__
        in the *AppConfig User Guide*.

        :param application_id: The application ID.
        :param name: A name for the configuration profile.
        :param location_uri: A URI to locate the configuration.
        :param description: A description of the configuration profile.
        :param retrieval_role_arn: The ARN of an IAM role with permission to access the configuration at
        the specified ``LocationUri``.
        :param validators: A list of methods for validating the configuration.
        :param tags: Metadata to assign to the configuration profile.
        :param type: The type of configurations contained in the profile.
        :returns: ConfigurationProfile
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("CreateDeploymentStrategy")
    def create_deployment_strategy(
        self,
        context: RequestContext,
        name: Name,
        deployment_duration_in_minutes: MinutesBetween0And24Hours,
        growth_factor: GrowthFactor,
        replicate_to: ReplicateTo,
        description: Description = None,
        final_bake_time_in_minutes: MinutesBetween0And24Hours = None,
        growth_type: GrowthType = None,
        tags: TagMap = None,
    ) -> DeploymentStrategy:
        """Creates a deployment strategy that defines important criteria for
        rolling out your configuration to the designated targets. A deployment
        strategy includes the overall duration required, a percentage of targets
        to receive the deployment during each interval, an algorithm that
        defines how percentage grows, and bake time.

        :param name: A name for the deployment strategy.
        :param deployment_duration_in_minutes: Total amount of time for a deployment to last.
        :param growth_factor: The percentage of targets to receive a deployed configuration during
        each interval.
        :param replicate_to: Save the deployment strategy to a Systems Manager (SSM) document.
        :param description: A description of the deployment strategy.
        :param final_bake_time_in_minutes: The amount of time AppConfig monitors for alarms before considering the
        deployment to be complete and no longer eligible for automatic roll
        back.
        :param growth_type: The algorithm used to define how percentage grows over time.
        :param tags: Metadata to assign to the deployment strategy.
        :returns: DeploymentStrategy
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("CreateEnvironment")
    def create_environment(
        self,
        context: RequestContext,
        application_id: Id,
        name: Name,
        description: Description = None,
        monitors: MonitorList = None,
        tags: TagMap = None,
    ) -> Environment:
        """Creates an environment. For each application, you define one or more
        environments. An environment is a logical deployment group of AppConfig
        targets, such as applications in a ``Beta`` or ``Production``
        environment. You can also define environments for application
        subcomponents such as the ``Web``, ``Mobile`` and ``Back-end``
        components for your application. You can configure Amazon CloudWatch
        alarms for each environment. The system monitors alarms during a
        configuration deployment. If an alarm is triggered, the system rolls
        back the configuration.

        :param application_id: The application ID.
        :param name: A name for the environment.
        :param description: A description of the environment.
        :param monitors: Amazon CloudWatch alarms to monitor during the deployment process.
        :param tags: Metadata to assign to the environment.
        :returns: Environment
        :raises InternalServerException:
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("CreateHostedConfigurationVersion")
    def create_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        content: Blob,
        content_type: StringWithLengthBetween1And255,
        description: Description = None,
        latest_version_number: Integer = None,
    ) -> HostedConfigurationVersion:
        """Creates a new configuration in the AppConfig hosted configuration store.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param content: The content of the configuration or the configuration data.
        :param content_type: A standard MIME type describing the format of the configuration content.
        :param description: A description of the configuration.
        :param latest_version_number: An optional locking token used to prevent race conditions from
        overwriting configuration updates when creating a new version.
        :returns: HostedConfigurationVersion
        :raises BadRequestException:
        :raises ServiceQuotaExceededException:
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises PayloadTooLargeException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("DeleteApplication")
    def delete_application(self, context: RequestContext, application_id: Id) -> None:
        """Deletes an application. Deleting an application does not delete a
        configuration from a host.

        :param application_id: The ID of the application to delete.
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteConfigurationProfile")
    def delete_configuration_profile(
        self, context: RequestContext, application_id: Id, configuration_profile_id: Id
    ) -> None:
        """Deletes a configuration profile. Deleting a configuration profile does
        not delete a configuration from a host.

        :param application_id: The application ID that includes the configuration profile you want to
        delete.
        :param configuration_profile_id: The ID of the configuration profile you want to delete.
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteDeploymentStrategy")
    def delete_deployment_strategy(
        self, context: RequestContext, deployment_strategy_id: DeploymentStrategyId
    ) -> None:
        """Deletes a deployment strategy. Deleting a deployment strategy does not
        delete a configuration from a host.

        :param deployment_strategy_id: The ID of the deployment strategy you want to delete.
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteEnvironment")
    def delete_environment(
        self, context: RequestContext, application_id: Id, environment_id: Id
    ) -> None:
        """Deletes an environment. Deleting an environment does not delete a
        configuration from a host.

        :param application_id: The application ID that includes the environment that you want to
        delete.
        :param environment_id: The ID of the environment that you want to delete.
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteHostedConfigurationVersion")
    def delete_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        version_number: Integer,
    ) -> None:
        """Deletes a version of a configuration from the AppConfig hosted
        configuration store.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param version_number: The versions number to delete.
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("GetApplication")
    def get_application(self, context: RequestContext, application_id: Id) -> Application:
        """Retrieves information about an application.

        :param application_id: The ID of the application you want to get.
        :returns: Application
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetConfiguration")
    def get_configuration(
        self,
        context: RequestContext,
        application: StringWithLengthBetween1And64,
        environment: StringWithLengthBetween1And64,
        configuration: StringWithLengthBetween1And64,
        client_id: StringWithLengthBetween1And64,
        client_configuration_version: Version = None,
    ) -> Configuration:
        """Retrieves the latest deployed configuration.

        Note the following important information.

        -  This API action has been deprecated. Calls to receive configuration
           data should use the
           `StartConfigurationSession <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/API_appconfigdata_StartConfigurationSession.html>`__
           and
           `GetLatestConfiguration <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/API_appconfigdata_GetLatestConfiguration.html>`__
           APIs instead.

        -  ``GetConfiguration`` is a priced call. For more information, see
           `Pricing <https://aws.amazon.com/systems-manager/pricing/>`__.

        -  AppConfig uses the value of the ``ClientConfigurationVersion``
           parameter to identify the configuration version on your clients. If
           you don’t send ``ClientConfigurationVersion`` with each call to
           ``GetConfiguration``, your clients receive the current configuration.
           You are charged each time your clients receive a configuration.

           To avoid excess charges, we recommend you use the
           `StartConfigurationSession <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/StartConfigurationSession.html>`__
           and
           `GetLatestConfiguration <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/GetLatestConfiguration.html>`__
           APIs, which track the client configuration version on your behalf. If
           you choose to continue using ``GetConfiguration``, we recommend that
           you include the ``ClientConfigurationVersion`` value with every call
           to ``GetConfiguration``. The value to use for
           ``ClientConfigurationVersion`` comes from the
           ``ConfigurationVersion`` attribute returned by ``GetConfiguration``
           when there is new or updated data, and should be saved for subsequent
           calls to ``GetConfiguration``.

        :param application: The application to get.
        :param environment: The environment to get.
        :param configuration: The configuration to get.
        :param client_id: The clientId parameter in the following command is a unique,
        user-specified ID to identify the client for the configuration.
        :param client_configuration_version: The configuration version returned in the most recent
        ``GetConfiguration`` response.
        :returns: Configuration
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetConfigurationProfile")
    def get_configuration_profile(
        self, context: RequestContext, application_id: Id, configuration_profile_id: Id
    ) -> ConfigurationProfile:
        """Retrieves information about a configuration profile.

        :param application_id: The ID of the application that includes the configuration profile you
        want to get.
        :param configuration_profile_id: The ID of the configuration profile that you want to get.
        :returns: ConfigurationProfile
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetDeployment")
    def get_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_number: Integer,
    ) -> Deployment:
        """Retrieves information about a configuration deployment.

        :param application_id: The ID of the application that includes the deployment you want to get.
        :param environment_id: The ID of the environment that includes the deployment you want to get.
        :param deployment_number: The sequence number of the deployment.
        :returns: Deployment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetDeploymentStrategy")
    def get_deployment_strategy(
        self, context: RequestContext, deployment_strategy_id: DeploymentStrategyId
    ) -> DeploymentStrategy:
        """Retrieves information about a deployment strategy. A deployment strategy
        defines important criteria for rolling out your configuration to the
        designated targets. A deployment strategy includes the overall duration
        required, a percentage of targets to receive the deployment during each
        interval, an algorithm that defines how percentage grows, and bake time.

        :param deployment_strategy_id: The ID of the deployment strategy to get.
        :returns: DeploymentStrategy
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetEnvironment")
    def get_environment(
        self, context: RequestContext, application_id: Id, environment_id: Id
    ) -> Environment:
        """Retrieves information about an environment. An environment is a logical
        deployment group of AppConfig applications, such as applications in a
        ``Production`` environment or in an ``EU_Region`` environment. Each
        configuration deployment targets an environment. You can enable one or
        more Amazon CloudWatch alarms for an environment. If an alarm is
        triggered during a deployment, AppConfig roles back the configuration.

        :param application_id: The ID of the application that includes the environment you want to get.
        :param environment_id: The ID of the environment that you want to get.
        :returns: Environment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetHostedConfigurationVersion")
    def get_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        version_number: Integer,
    ) -> HostedConfigurationVersion:
        """Retrieves information about a specific configuration version.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param version_number: The version.
        :returns: HostedConfigurationVersion
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ListApplications")
    def list_applications(
        self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
    ) -> Applications:
        """Lists all applications in your Amazon Web Services account.

        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: Applications
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListConfigurationProfiles", expand=False)
    def list_configuration_profiles(
        self, context: RequestContext, request: ListConfigurationProfilesRequest
    ) -> ConfigurationProfiles:
        """Lists the configuration profiles for an application.

        :param application_id: The application ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :param type: A filter based on the type of configurations that the configuration
        profile contains.
        :returns: ConfigurationProfiles
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListDeploymentStrategies")
    def list_deployment_strategies(
        self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
    ) -> DeploymentStrategies:
        """Lists deployment strategies.

        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: DeploymentStrategies
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListDeployments")
    def list_deployments(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> Deployments:
        """Lists the deployments for an environment in descending deployment number
        order.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param max_results: The maximum number of items that may be returned for this call.
        :param next_token: The token returned by a prior call to this operation indicating the next
        set of results to be returned.
        :returns: Deployments
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListEnvironments")
    def list_environments(
        self,
        context: RequestContext,
        application_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> Environments:
        """Lists the environments for an application.

        :param application_id: The application ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: Environments
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListHostedConfigurationVersions")
    def list_hosted_configuration_versions(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> HostedConfigurationVersions:
        """Lists configurations stored in the AppConfig hosted configuration store
        by version.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: HostedConfigurationVersions
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(self, context: RequestContext, resource_arn: Arn) -> ResourceTags:
        """Retrieves the list of key-value tags assigned to the resource.

        :param resource_arn: The resource ARN.
        :returns: ResourceTags
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("StartDeployment")
    def start_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_strategy_id: DeploymentStrategyId,
        configuration_profile_id: Id,
        configuration_version: Version,
        description: Description = None,
        tags: TagMap = None,
    ) -> Deployment:
        """Starts a deployment.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param deployment_strategy_id: The deployment strategy ID.
        :param configuration_profile_id: The configuration profile ID.
        :param configuration_version: The configuration version to deploy.
        :param description: A description of the deployment.
        :param tags: Metadata to assign to the deployment.
        :returns: Deployment
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("StopDeployment")
    def stop_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_number: Integer,
    ) -> Deployment:
        """Stops a deployment. This API action works only on deployments that have
        a status of ``DEPLOYING``. This action moves the deployment to a status
        of ``ROLLED_BACK``.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param deployment_number: The sequence number of the deployment.
        :returns: Deployment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(self, context: RequestContext, resource_arn: Arn, tags: TagMap) -> None:
        """Assigns metadata to an AppConfig resource. Tags help organize and
        categorize your AppConfig resources. Each tag consists of a key and an
        optional value, both of which you define. You can specify a maximum of
        50 tags for a resource.

        :param resource_arn: The ARN of the resource for which to retrieve tags.
        :param tags: The key-value string map.
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: Arn, tag_keys: TagKeyList
    ) -> None:
        """Deletes a tag key and value from an AppConfig resource.

        :param resource_arn: The ARN of the resource for which to remove tags.
        :param tag_keys: The tag keys to delete.
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateApplication")
    def update_application(
        self,
        context: RequestContext,
        application_id: Id,
        name: Name = None,
        description: Description = None,
    ) -> Application:
        """Updates an application.

        :param application_id: The application ID.
        :param name: The name of the application.
        :param description: A description of the application.
        :returns: Application
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateConfigurationProfile")
    def update_configuration_profile(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        name: Name = None,
        description: Description = None,
        retrieval_role_arn: RoleArn = None,
        validators: ValidatorList = None,
    ) -> ConfigurationProfile:
        """Updates a configuration profile.

        :param application_id: The application ID.
        :param configuration_profile_id: The ID of the configuration profile.
        :param name: The name of the configuration profile.
        :param description: A description of the configuration profile.
        :param retrieval_role_arn: The ARN of an IAM role with permission to access the configuration at
        the specified ``LocationUri``.
        :param validators: A list of methods for validating the configuration.
        :returns: ConfigurationProfile
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateDeploymentStrategy")
    def update_deployment_strategy(
        self,
        context: RequestContext,
        deployment_strategy_id: DeploymentStrategyId,
        description: Description = None,
        deployment_duration_in_minutes: MinutesBetween0And24Hours = None,
        final_bake_time_in_minutes: MinutesBetween0And24Hours = None,
        growth_factor: GrowthFactor = None,
        growth_type: GrowthType = None,
    ) -> DeploymentStrategy:
        """Updates a deployment strategy.

        :param deployment_strategy_id: The deployment strategy ID.
        :param description: A description of the deployment strategy.
        :param deployment_duration_in_minutes: Total amount of time for a deployment to last.
        :param final_bake_time_in_minutes: The amount of time that AppConfig monitors for alarms before considering
        the deployment to be complete and no longer eligible for automatic
        rollback.
        :param growth_factor: The percentage of targets to receive a deployed configuration during
        each interval.
        :param growth_type: The algorithm used to define how percentage grows over time.
        :returns: DeploymentStrategy
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateEnvironment")
    def update_environment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        name: Name = None,
        description: Description = None,
        monitors: MonitorList = None,
    ) -> Environment:
        """Updates an environment.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param name: The name of the environment.
        :param description: A description of the environment.
        :param monitors: Amazon CloudWatch alarms to monitor during the deployment process.
        :returns: Environment
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ValidateConfiguration")
    def validate_configuration(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        configuration_version: Version,
    ) -> None:
        """Uses the validators in a configuration profile to validate a
        configuration.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param configuration_version: The version of the configuration to validate.
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError
